@@ -29,6 +29,10 @@
 #include <VBox/VBoxGuestLib.h>
 #include <VBoxDisplay.h>
 
+#ifdef VBOX_WITH_HGSMI
+#include <iprt/initterm.h>
+#endif
+
 #if _MSC_VER >= 1400 /* bird: MS fixed swprintf to be standard-conforming... */
 #define _INC_SWPRINTF_INL_
 extern "C" int __cdecl swprintf(wchar_t *, const wchar_t *, ...);
@@ -59,6 +63,10 @@ ULONG DriverEntry(IN PVOID Context1, IN PVOID Context2)
     VIDEO_HW_INITIALIZATION_DATA InitData;
     ULONG rc;
 
+#ifdef VBOX_WITH_HGSMI
+    RTR0Init(0);
+#endif
+
     dprintf(("VBoxVideo::DriverEntry. Built %s %s\n", __DATE__, __TIME__));
 
     VideoPortZeroMemory(&InitData, sizeof(VIDEO_HW_INITIALIZATION_DATA));
@@ -1206,7 +1214,10 @@ VP_STATUS VBoxVideoFindAdapter(IN PVOID HwDeviceExtension,
       /* Setup the Device Extension and if possible secondary displays. */
       VBoxSetupDisplays((PDEVICE_EXTENSION)HwDeviceExtension, ConfigInfo, AdapterMemorySize);
 #else
-      /* Guest supports only HGSMI, the old VBVA via VMMDev is not supported. Old 
+      /* Initialize VBoxGuest library, which is used for requests which go through VMMDev. */
+      rc = VbglInit ();
+
+      /* Guest supports only HGSMI, the old VBVA via VMMDev is not supported. Old
        * code will be ifdef'ed and later removed.
        * The host will however support both old and new interface to keep compatibility
        * with old guest additions.
@@ -1277,6 +1288,39 @@ BOOLEAN VBoxVideoInitialize(PVOID HwDeviceExtension)
    return TRUE;
 }
 
+/**
+ * Send a request to the host to make the absolute pointer visible
+ */
+static BOOLEAN ShowPointer(PVOID HwDeviceExtension)
+{
+    LogRelFlowFunc(("\n"));
+    BOOLEAN Result;
+
+    if (DEV_MOUSE_HIDDEN((PDEVICE_EXTENSION)HwDeviceExtension))
+    {
+        // tell the host to use the guest's pointer
+        VIDEO_POINTER_ATTRIBUTES PointerAttributes;
+
+        /* Visible and No Shape means Show the pointer.
+         * It is enough to init only this field.
+         */
+        PointerAttributes.Enable = VBOX_MOUSE_POINTER_VISIBLE;
+
+#ifndef VBOX_WITH_HGSMI
+        Result = vboxUpdatePointerShape(&PointerAttributes, sizeof (PointerAttributes));
+#else
+        Result = vboxUpdatePointerShape(((PDEVICE_EXTENSION)HwDeviceExtension), &PointerAttributes, sizeof (PointerAttributes));
+#endif /* VBOX_WITH_HGSMI */
+
+        if (Result)
+            DEV_SET_MOUSE_SHOWN((PDEVICE_EXTENSION)HwDeviceExtension);
+        else
+            dprintf(("VBoxVideo::ShowPointer: Could not show the hardware pointer -> fallback\n"));
+    }
+    LogRelFlowFunc(("returning %d\n", Result));
+    return Result;
+}
+
 /**
  * VBoxVideoStartIO
  *
@@ -1487,21 +1531,10 @@ BOOLEAN VBoxVideoStartIO(PVOID HwDeviceExtension,
         {
             dprintf(("VBoxVideo::VBoxVideoStartIO: IOCTL_VIDEO_ENABLE_POINTER\n"));
             // find out whether the host wants absolute positioning
+            /// @todo this is now obsolete - remove it?
             if (vboxQueryHostWantsAbsolute())
-            {
-                // tell the host to use the guest's pointer
-                VIDEO_POINTER_ATTRIBUTES PointerAttributes;
-
-                /* Visible and No Shape means Show the pointer.
-                 * It is enough to init only this field.
-                 */
-                PointerAttributes.Enable = VBOX_MOUSE_POINTER_VISIBLE;
-
-                Result = vboxUpdatePointerShape(&PointerAttributes, sizeof (PointerAttributes));
-
-                if (!Result)
-                    dprintf(("VBoxVideo::VBoxVideoStartIO: Could not hide hardware pointer -> fallback\n"));
-            } else
+                Result = ShowPointer(HwDeviceExtension);
+            else
             {
                 // fallback to software pointer
                 RequestPacket->StatusBlock->Status = ERROR_INVALID_FUNCTION;
@@ -1525,9 +1558,15 @@ BOOLEAN VBoxVideoStartIO(PVOID HwDeviceExtension,
                  */
                 PointerAttributes.Enable = 0;
 
+#ifndef VBOX_WITH_HGSMI
                 Result = vboxUpdatePointerShape(&PointerAttributes, sizeof (PointerAttributes));
+#else
+                Result = vboxUpdatePointerShape((PDEVICE_EXTENSION)HwDeviceExtension, &PointerAttributes, sizeof (PointerAttributes));
+#endif/* VBOX_WITH_HGSMI */
 
-                if (!Result)
+                if (Result)
+                    DEV_SET_MOUSE_HIDDEN((PDEVICE_EXTENSION)HwDeviceExtension);
+                else
                     dprintf(("VBoxVideo::VBoxVideoStartIO: Could not hide hardware pointer -> fallback\n"));
             } else
             {
@@ -1568,7 +1607,11 @@ BOOLEAN VBoxVideoStartIO(PVOID HwDeviceExtension,
                          pPointerAttributes->Row));
                 dprintf(("\tBytes attached: %d\n", RequestPacket->InputBufferLength - sizeof(VIDEO_POINTER_ATTRIBUTES)));
 #endif
+#ifndef VBOX_WITH_HGSMI
                 Result = vboxUpdatePointerShape(pPointerAttributes, RequestPacket->InputBufferLength);
+#else
+                Result = vboxUpdatePointerShape((PDEVICE_EXTENSION)HwDeviceExtension, pPointerAttributes, RequestPacket->InputBufferLength);
+#endif/* VBOX_WITH_HGSMI */
                 if (!Result)
                     dprintf(("VBoxVideo::VBoxVideoStartIO: Could not set hardware pointer -> fallback\n"));
             } else
@@ -1593,18 +1636,11 @@ BOOLEAN VBoxVideoStartIO(PVOID HwDeviceExtension,
         // set the pointer position
         case IOCTL_VIDEO_SET_POINTER_POSITION:
         {
-            /// @todo There is an issue when we disable pointer integration.
-            // The guest pointer will be invisible. We have to somehow cause
-            // the pointer attributes to be set again. But how? The same holds
-            // true for the opposite case where we get two pointers.
-
-            //dprintf(("VBoxVideo::VBoxVideoStartIO: IOCTL_VIDEO_SET_POINTER_POSITION\n"));
             // find out whether the host wants absolute positioning
+            /// @todo this is now obsolete - remove it?
             if (vboxQueryHostWantsAbsolute())
-            {
-                // @todo we are supposed to show currently invisible pointer?
-                Result = TRUE;
-            } else
+                Result = ShowPointer(HwDeviceExtension);
+            else
             {
                 // fallback to software pointer
                 RequestPacket->StatusBlock->Status = ERROR_INVALID_FUNCTION;
@@ -1851,6 +1887,74 @@ BOOLEAN VBoxVideoStartIO(PVOID HwDeviceExtension,
 
             break;
         }
+        case IOCTL_VIDEO_HGSMI_QUERY_CALLBACKS:
+        {
+            dprintf(("VBoxVideo::VBoxVideoStartIO: IOCTL_VIDEO_HGSMI_QUERY_CALLBACKS\n"));
+
+            if (RequestPacket->OutputBufferLength < sizeof(HGSMIQUERYCALLBACKS))
+            {
+                dprintf(("VBoxVideo::VBoxVideoStartIO: Output buffer too small: %d needed: %d!!!\n",
+                         RequestPacket->OutputBufferLength, sizeof(HGSMIQUERYCALLBACKS)));
+                RequestPacket->StatusBlock->Status = ERROR_INSUFFICIENT_BUFFER;
+                return FALSE;
+            }
+
+            if (!pDevExt->pPrimary->u.primary.bHGSMI)
+            {
+                RequestPacket->StatusBlock->Status = ERROR_INVALID_FUNCTION;
+                return FALSE;
+            }
+
+            HGSMIQUERYCALLBACKS *pInfo = (HGSMIQUERYCALLBACKS *)RequestPacket->OutputBuffer;
+
+            pInfo->hContext = pDevExt;
+            pInfo->pfnCompletionHandler = hgsmiHostCmdComplete;
+            pInfo->pfnRequestCommandsHandler = hgsmiHostCmdRequest;
+
+            RequestPacket->StatusBlock->Information = sizeof(HGSMIQUERYCALLBACKS);
+            Result = TRUE;
+            break;
+        }
+        case IOCTL_VIDEO_HGSMI_HANDLER_ENABLE:
+        {
+            dprintf(("VBoxVideo::VBoxVideoStartIO: IOCTL_VIDEO_HGSMI_HANDLER_ENABLE\n"));
+
+            if (RequestPacket->InputBufferLength< sizeof(HGSMIHANDLERENABLE))
+            {
+                dprintf(("VBoxVideo::VBoxVideoStartIO: Output buffer too small: %d needed: %d!!!\n",
+                         RequestPacket->InputBufferLength, sizeof(HGSMIHANDLERENABLE)));
+                RequestPacket->StatusBlock->Status = ERROR_INSUFFICIENT_BUFFER;
+                return FALSE;
+            }
+
+            if (!pDevExt->pPrimary->u.primary.bHGSMI)
+            {
+                RequestPacket->StatusBlock->Status = ERROR_INVALID_FUNCTION;
+                return FALSE;
+            }
+
+            HGSMIHANDLERENABLE *pInfo = (HGSMIHANDLERENABLE *)RequestPacket->InputBuffer;
+
+            int rc = vboxVBVAChannelDisplayEnable(pDevExt->pPrimary,
+                    pDevExt->iDevice,
+                    pInfo->u8Channel);
+            if(RT_FAILURE(rc))
+            {
+                RequestPacket->StatusBlock->Status = ERROR_INVALID_NAME;
+            }
+            Result = TRUE;
+            break;
+        }
+        case IOCTL_VIDEO_HGSMI_HANDLER_DISABLE:
+        {
+            /* TODO: implement */
+            if (!pDevExt->pPrimary->u.primary.bHGSMI)
+            {
+                RequestPacket->StatusBlock->Status = ERROR_INVALID_FUNCTION;
+                return FALSE;
+            }
+            break;
+        }
 #endif /* VBOX_WITH_HGSMI */
 
         default:
