@@ -26,6 +26,7 @@
 
 #ifdef VBOX_WITH_HGSMI
 #include <VBox/HGSMI/HGSMI.h>
+#include "VBoxHGSMI.h"
 #endif /* VBOX_WITH_HGSMI */
 
 #define IOCTL_VIDEO_INTERPRET_DISPLAY_MEMORY \
@@ -47,6 +48,16 @@
 #ifdef VBOX_WITH_HGSMI
 #define IOCTL_VIDEO_QUERY_HGSMI_INFO \
     CTL_CODE(FILE_DEVICE_VIDEO, 0x430, METHOD_BUFFERED, FILE_ANY_ACCESS)
+
+#define IOCTL_VIDEO_HGSMI_QUERY_CALLBACKS \
+    CTL_CODE(FILE_DEVICE_VIDEO, 0x431, METHOD_BUFFERED, FILE_ANY_ACCESS)
+
+#define IOCTL_VIDEO_HGSMI_HANDLER_ENABLE \
+    CTL_CODE(FILE_DEVICE_VIDEO, 0x432, METHOD_BUFFERED, FILE_ANY_ACCESS)
+
+#define IOCTL_VIDEO_HGSMI_HANDLER_DISABLE \
+    CTL_CODE(FILE_DEVICE_VIDEO, 0x433, METHOD_BUFFERED, FILE_ANY_ACCESS)
+
 #endif /* VBOX_WITH_HGSMI */
 
 #pragma pack(1)
@@ -108,6 +119,36 @@ typedef struct _QUERYHGSMIRESULT
     /* Minimum size of the VBAV buffer. */
     uint32_t u32MinVBVABufferSize;
 } QUERYHGSMIRESULT;
+
+/**
+ * Data returned by IOCTL_VIDEO_HGSMI_QUERY_CALLBACKS.
+ *
+ */
+typedef struct _HGSMIQUERYCALLBACKS
+{
+    HVBOXVIDEOHGSMI hContext;
+    PFNVBOXVIDEOHGSMICOMPLETION pfnCompletionHandler;
+    PFNVBOXVIDEOHGSMICOMMANDS   pfnRequestCommandsHandler;
+} HGSMIQUERYCALLBACKS;
+
+/**
+ * Data returned by IOCTL_VIDEO_HGSMI_HANDLER_ENABLE.
+ *
+ */
+typedef struct _HGSMIHANDLERENABLE
+{
+    uint8_t u8Channel;
+} HGSMIHANDLERENABLE;
+
+/**
+ * Data passed by IOCTL_VIDEO_HGSMI_HANDLER_DISABLE.
+ *
+ */
+typedef struct _HGSMIHANDLERDISABLE
+{
+    uint8_t u8Channel;
+} HGSMIHANDLERDISABLE;
+
 #endif /* VBOX_WITH_HGSMI */
 #pragma pack()
 
