@@ -325,7 +325,7 @@ HRESULT WINAPI VarR8FromUI4(ULONG,double*);
 HRESULT WINAPI VarR8FromUI8(ULONG64,double*);
 HRESULT WINAPI VarR8FromStr(OLECHAR*,LCID,ULONG,double*);
 HRESULT WINAPI VarR8FromCy(CY,double*);
-HRESULT WINAPI VarR8FromDec(DECIMAL*,double*);
+HRESULT WINAPI VarR8FromDec(const DECIMAL*,double*);
 HRESULT WINAPI VarR8FromDisp(IDispatch*,LCID,double*);
 
 HRESULT WINAPI VarDateFromUI1(BYTE,DATE*);
