@@ -549,6 +549,7 @@ BOOLAPI InternetUnlockRequestFile(HANDLE);
 #define INTERNET_OPTION_ERROR_MASK              62
 #define INTERNET_OPTION_FROM_CACHE_TIMEOUT      63
 #define INTERNET_OPTION_BYPASS_EDITED_ENTRY     64
+#define INTERNET_OPTION_HTTP_DECODING           65
 #define INTERNET_OPTION_DIAGNOSTIC_SOCKET_INFO  67
 #define INTERNET_OPTION_CODEPAGE                68
 #define INTERNET_OPTION_CACHE_TIMESTAMPS        69
@@ -580,6 +581,10 @@ BOOLAPI InternetUnlockRequestFile(HANDLE);
 #define INTERNET_OPTION_PROXY_SETTINGS_CHANGED  95
 #define INTERNET_OPTION_DATAFILE_EXT            96
 
+#define INTERNET_OPTION_CODEPAGE_PATH           100
+#define INTERNET_OPTION_CODEPAGE_EXTRA          101
+#define INTERNET_OPTION_IDN                     102
+
 #define INTERNET_FIRST_OPTION                   INTERNET_OPTION_CALLBACK
 #define INTERNET_LAST_OPTION                    INTERNET_OPTION_DATAFILE_EXT
 
@@ -1360,6 +1365,23 @@ INTERNETAPI DWORD WINAPI InternetConfirmZoneCrossingA(HWND ,LPSTR ,LPSTR ,BOOL);
 INTERNETAPI DWORD WINAPI InternetConfirmZoneCrossingW(HWND ,LPWSTR ,LPWSTR ,BOOL);
 #define InternetConfirmZoneCrossing WINELIB_NAME_AW(InternetConfirmZoneCrossing)
 
+#define PRIVACY_TEMPLATE_NO_COOKIES  0
+#define PRIVACY_TEMPLATE_HIGH        1
+#define PRIVACY_TEMPLATE_MEDIUM_HIGH 2
+#define PRIVACY_TEMPLATE_MEDIUM      3
+#define PRIVACY_TEMPLATE_MEDIUM_LOW  4
+#define PRIVACY_TEMPLATE_LOW         5
+#define PRIVACY_TEMPLATE_CUSTOM      100
+#define PRIVACY_TEMPLATE_ADVANCED    101
+
+#define PRIVACY_TEMPLATE_MAX         PRIVACY_TEMPLATE_LOW
+
+#define PRIVACY_TYPE_FIRST_PARTY 0
+#define PRIVACY_TYPE_THIRD_PARTY 1
+
+INTERNETAPI DWORD WINAPI PrivacySetZonePreferenceW(DWORD,DWORD,DWORD,LPCWSTR);
+INTERNETAPI DWORD WINAPI PrivacyGetZonePreferenceW(DWORD,DWORD,LPDWORD,LPWSTR,LPDWORD);
+
 #define INTERNET_ERROR_BASE                     12000
 
 #define ERROR_INTERNET_OUT_OF_HANDLES           (INTERNET_ERROR_BASE + 1)
@@ -1451,7 +1473,10 @@ INTERNETAPI DWORD WINAPI InternetConfirmZoneCrossingW(HWND ,LPWSTR ,LPWSTR ,BOOL
 #define ERROR_INTERNET_SEC_CERT_REVOKED    (INTERNET_ERROR_BASE + 170)
 #define ERROR_INTERNET_FAILED_DUETOSECURITYCHECK  (INTERNET_ERROR_BASE + 171)
 #define ERROR_INTERNET_NOT_INITIALIZED            (INTERNET_ERROR_BASE + 172)
-#define INTERNET_ERROR_LAST                       ERROR_INTERNET_NOT_INITIALIZED
+#define ERROR_INTERNET_NEED_MSN_SSPI_PKG          (INTERNET_ERROR_BASE + 173)
+#define ERROR_INTERNET_LOGIN_FAILURE_DISPLAY_ENTITY_BODY  (INTERNET_ERROR_BASE + 174)
+#define ERROR_INTERNET_DECODING_FAILED            (INTERNET_ERROR_BASE + 175)
+#define INTERNET_ERROR_LAST                       ERROR_INTERNET_DECODING_FAILED
 
 
 #define NORMAL_CACHE_ENTRY              0x00000001
