@@ -178,22 +178,23 @@ static char *next_dll_path( struct dll_path_context *context )
 
     switch(index)
     {
-    case 0:  /* try programs dir for .exe files */
-        if (!context->win16 && namelen > 4 && !memcmp( context->name + namelen - 4, ".exe", 4 ))
+    case 0:  /* try dlls dir with subdir prefix */
+        if (namelen > 4 && !memcmp( context->name + namelen - 4, ".dll", 4 )) namelen -= 4;
+        if (!context->win16) path = prepend( path, context->name, namelen );
+        path = prepend( path, "/dlls", sizeof("/dlls") - 1 );
+        path = prepend( path, build_dir, strlen(build_dir) );
+        return path;
+    case 1:  /* try programs dir with subdir prefix */
+        if (!context->win16)
         {
-            path = prepend( path, context->name, namelen - 4 );
+            if (namelen > 4 && !memcmp( context->name + namelen - 4, ".exe", 4 )) namelen -= 4;
+            path = prepend( path, context->name, namelen );
             path = prepend( path, "/programs", sizeof("/programs") - 1 );
             path = prepend( path, build_dir, strlen(build_dir) );
             return path;
         }
         context->index++;
         /* fall through */
-    case 1:  /* try dlls dir with subdir prefix */
-        if (namelen > 4 && !memcmp( context->name + namelen - 4, ".dll", 4 )) namelen -= 4;
-        if (!context->win16) path = prepend( path, context->name, namelen );
-        path = prepend( path, "/dlls", sizeof("/dlls") - 1 );
-        path = prepend( path, build_dir, strlen(build_dir) );
-        return path;
     default:
         index -= 2;
         if (index < nb_dll_paths)
@@ -448,6 +449,18 @@ static void *map_dll( const IMAGE_NT_HEADERS *nt_descr )
 }
 
 
+/***********************************************************************
+ *           __wine_get_main_environment
+ *
+ * Return an environment pointer to work around lack of environ variable.
+ * Only exported on Mac OS.
+ */
+char **__wine_get_main_environment(void)
+{
+    return environ;
+}
+
+
 /***********************************************************************
  *           __wine_dll_register
  *
@@ -646,7 +659,7 @@ void wine_init( int argc, char *argv[], char *error, int error_size )
     build_dll_path();
     __wine_main_argc = argc;
     __wine_main_argv = argv;
-    __wine_main_environ = environ;
+    __wine_main_environ = __wine_get_main_environment();
     mmap_init();
 
     for (path = first_dll_path( "ntdll.dll", 0, &context ); path; path = next_dll_path( &context ))
