@@ -35,22 +35,23 @@
 #  define _InterlockedCompareExchange    _InterlockedCompareExchange_StupidDDKVsCompilerCrap
 #  define _InterlockedAddLargeStatistic  _InterlockedAddLargeStatistic_StupidDDKVsCompilerCrap
 #  pragma warning(disable : 4163)
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 #  include <ntddk.h>
-__END_DECLS
+RT_C_DECLS_END
 #  pragma warning(default : 4163)
 #  undef  _InterlockedExchange
 #  undef  _InterlockedExchangeAdd
 #  undef  _InterlockedCompareExchange
 #  undef  _InterlockedAddLargeStatistic
 # else
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 #  include <ntddk.h>
-__END_DECLS
+RT_C_DECLS_END
 # endif
 #endif
 
 #include <iprt/spinlock.h>
+#include <iprt/memobj.h>
 
 #include <VBox/VBoxGuest.h>
 
@@ -178,6 +179,9 @@ typedef struct VBOXGUESTDEVEXT
     // size in bytes of the hypervisor mapping
     ULONG hypervisorMappingSize;
 
+    /* Patch memory object. */
+    RTR0MEMOBJ PatchMemObj;
+
     /* Physical address and length of VMMDev memory */
     PHYSICAL_ADDRESS memoryAddress;
     ULONG memoryLength;
