@@ -168,13 +168,13 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
 
         if (DisplayDevice.StateFlags & DISPLAY_DEVICE_PRIMARY_DEVICE)
         {
-            Log(("Found primary device. err %d\n", GetLastError ()));
+            Log(("ResizeDisplayDevice: Found primary device. err %d\n", GetLastError ()));
             NumDevices++;
         }
         else if (!(DisplayDevice.StateFlags & DISPLAY_DEVICE_MIRRORING_DRIVER))
         {
             
-            Log(("Found secondary device. err %d\n", GetLastError ()));
+            Log(("ResizeDisplayDevice: Found secondary device. err %d\n", GetLastError ()));
             NumDevices++;
         }
         
@@ -183,11 +183,11 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
         i++;
     }
     
-    Log(("Found total %d devices. err %d\n", NumDevices, GetLastError ()));
+    Log(("ResizeDisplayDevice: Found total %d devices. err %d\n", NumDevices, GetLastError ()));
     
     if (NumDevices == 0 || Id >= NumDevices)
     {
-        Log(("Requested identifier %d is invalid. err %d\n", Id, GetLastError ()));
+        Log(("ResizeDisplayDevice: Requested identifier %d is invalid. err %d\n", Id, GetLastError ()));
         return FALSE;
     }
     
@@ -205,20 +205,20 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
     i = 0;
     while (EnumDisplayDevices (NULL, i, &DisplayDevice, 0))
     { 
-        Log(("[%d(%d)] %s\n", i, DevNum, DisplayDevice.DeviceName));
+        Log(("ResizeDisplayDevice: [%d(%d)] %s\n", i, DevNum, DisplayDevice.DeviceName));
         
         BOOL bFetchDevice = FALSE;
 
         if (DisplayDevice.StateFlags & DISPLAY_DEVICE_PRIMARY_DEVICE)
         {
-            Log(("Found primary device. err %d\n", GetLastError ()));
+            Log(("ResizeDisplayDevice: Found primary device. err %d\n", GetLastError ()));
             DevPrimaryNum = DevNum;
             bFetchDevice = TRUE;
         }
         else if (!(DisplayDevice.StateFlags & DISPLAY_DEVICE_MIRRORING_DRIVER))
         {
             
-            Log(("Found secondary device. err %d\n", GetLastError ()));
+            Log(("ResizeDisplayDevice: Found secondary device. err %d\n", GetLastError ()));
             bFetchDevice = TRUE;
         }
         
@@ -226,7 +226,7 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
         {
             if (DevNum >= NumDevices)
             {
-                Log(("%d >= %d\n", NumDevices, DevNum));
+                Log(("ResizeDisplayDevice: %d >= %d\n", NumDevices, DevNum));
                 return FALSE;
             }
         
@@ -241,14 +241,14 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
             if (!EnumDisplaySettings((LPSTR)DisplayDevice.DeviceName,
                  ENUM_REGISTRY_SETTINGS, &paDeviceModes[DevNum]))
             {
-                Log(("EnumDisplaySettings err %d\n", GetLastError ()));
+                Log(("ResizeDisplayDevice: EnumDisplaySettings err %d\n", GetLastError ()));
                 return FALSE;
             }
             
             if (   paDeviceModes[DevNum].dmPelsWidth == 0
                 || paDeviceModes[DevNum].dmPelsHeight == 0)
             {
-                /* No ENUM_REGISTRY_SETTINGS yet. Seen on Windows 7 after installation.
+                /* No ENUM_REGISTRY_SETTINGS yet. Seen on Vista after installation.
                  * Get the current video mode then.
                  */
                 ZeroMemory(&paDeviceModes[DevNum], sizeof(DEVMODE));
@@ -256,12 +256,16 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
                 if (!EnumDisplaySettings((LPSTR)DisplayDevice.DeviceName,
                      ENUM_CURRENT_SETTINGS, &paDeviceModes[DevNum]))
                 {
+                    /* ENUM_CURRENT_SETTINGS returns FALSE when the display is not active:
+                     * for example a disabled secondary display.
+                     * Do not return here, ignore the error and set the display info to 0x0x0.
+                     */
                     Log(("EnumDisplaySettings(ENUM_CURRENT_SETTINGS) err %d\n", GetLastError ()));
-                    return FALSE;
+                    ZeroMemory(&paDeviceModes[DevNum], sizeof(DEVMODE));
                 }
             }
             
-            Log(("%dx%dx%d at %d,%d\n",
+            Log(("ResizeDisplayDevice: %dx%dx%d at %d,%d\n",
                     paDeviceModes[DevNum].dmPelsWidth,
                     paDeviceModes[DevNum].dmPelsHeight,
                     paDeviceModes[DevNum].dmBitsPerPel,
@@ -301,7 +305,7 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
         && paRects[Id].bottom - paRects[Id].top == Height
         && paDeviceModes[Id].dmBitsPerPel == BitsPerPixel)
     {
-        Log(("VBoxDisplayThread : already at desired resolution.\n"));
+        Log(("ResizeDisplayDevice: Already at desired resolution.\n"));
         return FALSE;
     }
 
@@ -309,7 +313,7 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
 #ifdef Log
     for (i = 0; i < NumDevices; i++)
     {
-        Log(("[%d]: %d,%d %dx%d\n",
+        Log(("ResizeDisplayDevice: [%d]: %d,%d %dx%d\n",
                 i, paRects[i].left, paRects[i].top,
                 paRects[i].right - paRects[i].left,
                 paRects[i].bottom - paRects[i].top));
@@ -332,7 +336,7 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
         paDeviceModes[i].dmPelsWidth  = paRects[i].right - paRects[i].left;
         paDeviceModes[i].dmPelsHeight = paRects[i].bottom - paRects[i].top;
         
-        /* On Windows 7 one must specify DM_BITSPERPEL.
+        /* On Vista one must specify DM_BITSPERPEL.
          * Note that the current mode dmBitsPerPel is already in the DEVMODE structure.
          */
         paDeviceModes[i].dmFields = DM_POSITION | DM_PELSHEIGHT | DM_PELSWIDTH | DM_BITSPERPEL;
@@ -344,7 +348,7 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
             paDeviceModes[i].dmBitsPerPel = BitsPerPixel;
         }
 
-        Log(("calling pfnChangeDisplaySettingsEx %x: %dx%dx%d at %d,%d\n",
+        Log(("ResizeDisplayDevice: pfnChangeDisplaySettingsEx %x: %dx%dx%d at %d,%d\n",
               gCtx.pfnChangeDisplaySettingsEx,
               paDeviceModes[i].dmPelsWidth,
               paDeviceModes[i].dmPelsHeight,
@@ -352,15 +356,14 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
               paDeviceModes[i].dmPosition.x,
               paDeviceModes[i].dmPosition.y));
 
-        gCtx.pfnChangeDisplaySettingsEx((LPSTR)paDisplayDevices[i].DeviceName, 
-                                        &paDeviceModes[i], NULL, CDS_NORESET | CDS_UPDATEREGISTRY, NULL); 
-
-        Log(("ChangeDisplaySettings position err %d\n", GetLastError ()));
+        gCtx.pfnChangeDisplaySettingsEx((LPSTR)paDisplayDevices[i].DeviceName,
+                                        &paDeviceModes[i], NULL, CDS_NORESET | CDS_UPDATEREGISTRY, NULL);
+        Log(("ResizeDisplayDevice: ChangeDisplaySettingsEx position err %d\n", GetLastError ()));
     }
     
     /* A second call to ChangeDisplaySettings updates the monitor. */
     LONG status = ChangeDisplaySettings(NULL, 0); 
-    Log(("ChangeDisplaySettings update status %d\n", status));
+    Log(("ResizeDisplayDevice: ChangeDisplaySettings update status %d\n", status));
     if (status == DISP_CHANGE_SUCCESSFUL || status == DISP_CHANGE_BADMODE)
     {
         /* Successfully set new video mode or our driver can not set the requested mode. Stop trying. */
@@ -383,7 +386,7 @@ unsigned __stdcall VBoxDisplayThread  (void *pInstance)
     VBoxGuestFilterMaskInfo maskInfo;
     DWORD cbReturned;
     
-    maskInfo.u32OrMask = VMMDEV_EVENT_DISPLAY_CHANGE_REQUEST;
+    maskInfo.u32OrMask = VMMDEV_EVENT_DISPLAY_CHANGE_REQUEST | VMMDEV_EVENT_MOUSE_CAPABILITIES_CHANGED;
     maskInfo.u32NotMask = 0;
     if (DeviceIoControl (gVBoxDriver, VBOXGUEST_IOCTL_CTL_FILTER_MASK, &maskInfo, sizeof (maskInfo), NULL, 0, &cbReturned, NULL))
     {
@@ -400,10 +403,10 @@ unsigned __stdcall VBoxDisplayThread  (void *pInstance)
         /* wait for a display change event */
         VBoxGuestWaitEventInfo waitEvent;
         waitEvent.u32TimeoutIn = 1000;
-        waitEvent.u32EventMaskIn = VMMDEV_EVENT_DISPLAY_CHANGE_REQUEST;
+        waitEvent.u32EventMaskIn = VMMDEV_EVENT_DISPLAY_CHANGE_REQUEST | VMMDEV_EVENT_MOUSE_CAPABILITIES_CHANGED;
         if (DeviceIoControl(gVBoxDriver, VBOXGUEST_IOCTL_WAITEVENT, &waitEvent, sizeof(waitEvent), &waitEvent, sizeof(waitEvent), &cbReturned, NULL))
         {
-            Log(("VBoxDisplayThread : DeviceIOControl succeded\n"));
+            /*Log(("VBoxDisplayThread : DeviceIOControl succeded\n"));*/
 
             if (NULL == pCtx) {
                 Log(("VBoxDisplayThread : Invalid context detected!\n"));
@@ -419,7 +422,7 @@ unsigned __stdcall VBoxDisplayThread  (void *pInstance)
             if (WaitForSingleObject(pCtx->pEnv->hStopEvent, 0) == WAIT_OBJECT_0)
                 break;
 
-            Log(("VBoxDisplayThread : checking event\n"));
+            /*Log(("VBoxDisplayThread : checking event\n"));*/
 
             /* did we get the right event? */
             if (waitEvent.u32EventFlagsOut & VMMDEV_EVENT_DISPLAY_CHANGE_REQUEST)
@@ -588,6 +591,8 @@ unsigned __stdcall VBoxDisplayThread  (void *pInstance)
                     }
                 }
             }
+            if (waitEvent.u32EventFlagsOut & VMMDEV_EVENT_MOUSE_CAPABILITIES_CHANGED)
+                VBoxServiceReloadCursor();
         } else
         {
             Log(("VBoxDisplayThread : error 0 from DeviceIoControl VBOXGUEST_IOCTL_WAITEVENT\n"));
@@ -601,7 +606,7 @@ unsigned __stdcall VBoxDisplayThread  (void *pInstance)
     } while (!fTerminate);
 
     maskInfo.u32OrMask = 0;
-    maskInfo.u32NotMask = VMMDEV_EVENT_DISPLAY_CHANGE_REQUEST;
+    maskInfo.u32NotMask = VMMDEV_EVENT_DISPLAY_CHANGE_REQUEST | VMMDEV_EVENT_MOUSE_CAPABILITIES_CHANGED;
     if (DeviceIoControl (gVBoxDriver, VBOXGUEST_IOCTL_CTL_FILTER_MASK, &maskInfo, sizeof (maskInfo), NULL, 0, &cbReturned, NULL))
     {
         Log(("VBoxDisplayThread : DeviceIOControl(CtlMask - not) succeeded\n"));
