@@ -22,19 +22,12 @@
 #ifndef ___VBoxServiceInternal_h
 #define ___VBoxServiceInternal_h
 
+#include <stdio.h>
 #ifdef RT_OS_WINDOWS
 # include <Windows.h>
-# include <tchar.h>   /**@todo just drop this, this will be compiled as UTF-8/ANSI. */
-# include <process.h> /**@todo what's this here for?  */
+# include <process.h> /* Needed for file version information. */
 #endif
 
-/** @todo just move this into the windows specific code, it's not needed
- *        here. */
-/** The service name. */
-#define VBOXSERVICE_NAME           "VBoxService"
-/** The friendly service name. */
-#define VBOXSERVICE_FRIENDLY_NAME  "VBoxService"
-
 /**
  * A service descriptor.
  */
@@ -97,39 +90,71 @@ typedef VBOXSERVICE *PVBOXSERVICE;
 /** Pointer to a const VBOXSERVICE. */
 typedef VBOXSERVICE const *PCVBOXSERVICE;
 
+#ifdef RT_OS_WINDOWS
+/** The service name (needed for mutex creation on Windows). */
+#define VBOXSERVICE_NAME          "VBoxService"
+/** The friendly service name. */
+#define VBOXSERVICE_FRIENDLY_NAME "VBoxService"
+/** The following constant may be defined by including NtStatus.h. */
+#define STATUS_SUCCESS ((NTSTATUS)0x00000000L)
+/** Structure for storing the looked up user information. */
+typedef struct
+{
+    WCHAR szUser [_MAX_PATH];
+    WCHAR szAuthenticationPackage [_MAX_PATH];
+    WCHAR szLogonDomain [_MAX_PATH];
+} VBOXSERVICEVMINFOUSER, *PVBOXSERVICEVMINFOUSER;
+/** Structure for the file information lookup. */
+typedef struct
+{
+    char* pszFilePath;
+    char* pszFileName;
+} VBOXSERVICEVMINFOFILE, *PVBOXSERVICEVMINFOFILE;
+/** Function prototypes for dynamic loading. */
+typedef DWORD (WINAPI* fnWTSGetActiveConsoleSessionId)();
+#endif
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 
 extern char *g_pszProgName;
 extern int g_cVerbosity;
 extern uint32_t g_DefaultInterval;
-/** Windows SCM stuff.
- *  @todo document each of them individually, this comment only documents
- *  g_vboxServiceStatusCode on windows. On the other platforms it will be
- *  dangling.
- *  @todo all this should be moved to -win.cpp and exposed via functions. */
-#ifdef RT_OS_WINDOWS
-extern DWORD                 g_vboxServiceStatusCode;   /** @todo g_rcWinService */
-extern SERVICE_STATUS_HANDLE g_vboxServiceStatusHandle; /** @todo g_hWinServiceStatus */
-extern SERVICE_TABLE_ENTRY const g_aServiceTable[];     /** @todo generate on the fly, see comment in main() from the enabled sub services. */
-#endif
 
 extern int VBoxServiceSyntax(const char *pszFormat, ...);
 extern int VBoxServiceError(const char *pszFormat, ...);
 extern void VBoxServiceVerbose(int iLevel, const char *pszFormat, ...);
 extern int VBoxServiceArgUInt32(int argc, char **argv, const char *psz, int *pi, uint32_t *pu32, uint32_t u32Min, uint32_t u32Max);
+extern unsigned VBoxServiceGetStartedServices(void);
 extern int VBoxServiceStartServices(unsigned iMain);
-extern void VBoxServiceStopServices(void);
-#ifdef RT_OS_WINDOWS
-extern int VBoxServiceWinInstall(void);
-extern int VBoxServiceWinUninstall(void);
-#endif
+extern int VBoxServiceStopServices(void);
 
 extern VBOXSERVICE g_TimeSync;
 extern VBOXSERVICE g_Clipboard;
 extern VBOXSERVICE g_Control;
+extern VBOXSERVICE g_VMInfo;
+extern VBOXSERVICE g_Exec;
 
-__END_DECLS
+#ifdef RT_OS_WINDOWS
+extern DWORD g_rcWinService;
+extern SERVICE_STATUS_HANDLE g_hWinServiceStatus;
+extern SERVICE_TABLE_ENTRY const g_aServiceTable[];     /** @todo generate on the fly, see comment in main() from the enabled sub services. */
+
+/** Installs the service into the registry. */
+extern int VBoxServiceWinInstall(void);
+/** Uninstalls the service from the registry. */
+extern int VBoxServiceWinUninstall(void);
+#ifdef VBOX_WITH_GUEST_PROPS
+/** Detects wheter a user is logged on based on the enumerated processes. */
+extern BOOL VboxServiceVMInfoWinIsLoggedIn(VBOXSERVICEVMINFOUSER* a_pUserInfo,
+                                           PLUID a_pSession,
+                                           PLUID a_pLuid,
+                                           DWORD a_dwNumOfProcLUIDs);
+/** Gets logon user IDs from enumerated processes. */
+extern DWORD VboxServiceVMInfoWinGetLUIDsFromProcesses(PLUID *ppLuid);
+#endif /* VBOX_WITH_GUEST_PROPS */
+#endif /* RT_OS_WINDOWS */
+
+RT_C_DECLS_END
 
 #endif
 
