@@ -42,16 +42,9 @@
 #endif
 
 #ifdef RT_OS_WINDOWS
-# ifdef VBOX_WITH_SIMPLIFIED_SLIRP_SYNC
 # define ICMP_SEND_ECHO(event, routine, addr, data, datasize, ipopt)                                        \
                 IcmpSendEcho2(pData->icmp_socket.sh, (event), NULL, NULL, (addr), (data), (datasize),       \
                                (ipopt), pData->pvIcmpBuffer, pData->szIcmpBuffer, 1)
-# else /* VBOX_WITH_SIMPLIFIED_SLIRP_SYNC */
-# define ICMP_SEND_ECHO(event, routine, addr, data, datasize, ipopt)                                                        \
-                IcmpSendEcho2(pData->icmp_socket.sh, NULL, (FARPROC)(routine), (void *)pData, (addr), (data), (datasize),   \
-                             (ipopt), pData->pvIcmpBuffer, pData->szIcmpBuffer, 1)
-static void WINAPI notify_slirp(void *);
-# endif /* !VBOX_WITH_SIMPLIFIED_SLIRP_SYNC */
 #endif /* RT_OS_WINDOWS */
 
 /* The message sent when emulating PING */
@@ -109,24 +102,18 @@ icmp_init(PNATState pData)
                                     GetProcAddress(pData->hmIcmpLibrary, "IcmpParseReplies");
         pData->pfIcmpCloseHandle = (BOOL (WINAPI *)(HANDLE))
                                     GetProcAddress(pData->hmIcmpLibrary, "IcmpCloseHandle");
-# ifdef VBOX_WITH_MULTI_DNS 
-        pData->pfGetAdaptersAddresses = (ULONG (WINAPI *)(HANDLE))
+        pData->pfGetAdaptersAddresses = (ULONG (WINAPI *)(ULONG, ULONG, PVOID, PIP_ADAPTER_ADDRESSES, PULONG))
                                     GetProcAddress(pData->hmIcmpLibrary, "GetAdaptersAddresses");
         if (pData->pfGetAdaptersAddresses == NULL) 
         {
             LogRel(("NAT: Can't find GetAdapterAddresses in Iphlpapi.dll"));
         }
-# endif
     }
 
     if (pData->pfIcmpParseReplies == NULL)
     {
-# ifdef VBOX_WITH_MULTI_DNS 
         if(pData->pfGetAdaptersAddresses == NULL) 
             FreeLibrary(pData->hmIcmpLibrary);
-# else
-        FreeLibrary(pData->hmIcmpLibrary);
-# endif
         pData->hmIcmpLibrary = LoadLibrary("Icmp.dll");
         if (pData->hmIcmpLibrary == NULL)
         {
@@ -151,9 +138,7 @@ icmp_init(PNATState pData)
         return 1;
     }
     pData->icmp_socket.sh = IcmpCreateFile();
-# ifdef VBOX_WITH_SIMPLIFIED_SLIRP_SYNC
     pData->phEvents[VBOX_ICMP_EVENT_INDEX] = CreateEvent(NULL, FALSE, FALSE, NULL);
-# endif /* VBOX_WITH_SIMPLIFIED_SLIRP_SYNC */
     pData->szIcmpBuffer = sizeof(ICMP_ECHO_REPLY) * 10;
     pData->pvIcmpBuffer = RTMemAlloc(pData->szIcmpBuffer);
 #endif /* RT_OS_WINDOWS */
@@ -321,7 +306,7 @@ icmp_input(PNATState pData, struct mbuf *m, int hlen)
 
     DEBUG_CALL("icmp_input");
     DEBUG_ARG("m = %lx", (long )m);
-    DEBUG_ARG("m_len = %d", m->m_len);
+    DEBUG_ARG("m_len = %d", m ? m->m_len : 0);
 
     icmpstat.icps_received++;
 
@@ -379,10 +364,6 @@ freeit:
                     switch (ntohl(ip->ip_dst.s_addr) & ~pData->netmask)
                     {
                         case CTL_DNS:
-#ifndef VBOX_WITH_MULTI_DNS
-                            addr.sin_addr = dns_addr;
-                            break;
-#endif
                         case CTL_ALIAS:
                         default:
                             addr.sin_addr = loopback_addr;
@@ -508,7 +489,7 @@ void icmp_error(PNATState pData, struct mbuf *msrc, u_char type, u_char code, in
 
     DEBUG_CALL("icmp_error");
     DEBUG_ARG("msrc = %lx", (long )msrc);
-    DEBUG_ARG("msrc_len = %d", msrc->m_len);
+    DEBUG_ARG("msrc_len = %d", msrc ? msrc->m_len : 0);
 
     if (type!=ICMP_UNREACH && type!=ICMP_TIMXCEED)
         goto end_error;
@@ -549,9 +530,10 @@ void icmp_error(PNATState pData, struct mbuf *msrc, u_char type, u_char code, in
         int new_m_size;
         m->m_data += if_maxlinkhdr;
         new_m_size = sizeof(struct ip) + ICMP_MINLEN + msrc->m_len + ICMP_MAXDATALEN;
-        if (new_m_size>m->m_size)
+        if (new_m_size > m->m_size)
             m_inc(m, new_m_size);
     }
+
     memcpy(m->m_data, msrc->m_data, msrc->m_len);
     m->m_len = msrc->m_len;                /* copy msrc to m */
 
@@ -623,7 +605,7 @@ void icmp_error(PNATState pData, struct mbuf *msrc, u_char type, u_char code, in
     return;
 
 end_error:
-    LogRel(("NAT: error occured while sending ICMP error message \n"));
+    LogRel(("NAT: error occurred while sending ICMP error message \n"));
 }
 #undef ICMP_MAXDATALEN
 
@@ -656,12 +638,3 @@ icmp_reflect(PNATState pData, struct mbuf *m)
 
     icmpstat.icps_reflect++;
 }
-#if defined(RT_OS_WINDOWS) && !defined(VBOX_WITH_SIMPLIFIED_SLIRP_SYNC)
-static void WINAPI
-notify_slirp(void *ctx)
-{
-    /* pData name is important see slirp_state.h */
-    PNATState pData = (PNATState)ctx;
-    fIcmp = 1;
-}
-#endif
