@@ -78,6 +78,7 @@ tcp_reass(PNATState pData, struct tcpcb *tp, struct tcphdr *th, int *tlenp, stru
     struct tseg_qent *te = NULL;
     struct socket *so = tp->t_socket;
     int flags;
+    STAM_PROFILE_START(&pData->StatTCP_reassamble, tcp_reassamble);
 
     /*
      * XXX: tcp_reass() is rather inefficient with its data structures
@@ -107,6 +108,7 @@ tcp_reass(PNATState pData, struct tcpcb *tp, struct tcphdr *th, int *tlenp, stru
         tcpstat.tcps_rcvmemdrop++;
         m_freem(pData, m);
         *tlenp = 0;
+        STAM_PROFILE_STOP(&pData->StatTCP_reassamble, tcp_reassamble);
         return (0);
     }
 
@@ -120,6 +122,7 @@ tcp_reass(PNATState pData, struct tcpcb *tp, struct tcphdr *th, int *tlenp, stru
         tcpstat.tcps_rcvmemdrop++;
         m_freem(pData, m);
         *tlenp = 0;
+        STAM_PROFILE_STOP(&pData->StatTCP_reassamble, tcp_reassamble);
         return (0);
     }
     tp->t_segqlen++;
@@ -217,10 +220,16 @@ present:
      * completed sequence space.
      */
     if (!TCPS_HAVEESTABLISHED(tp->t_state))
+    {
+        STAM_PROFILE_STOP(&pData->StatTCP_reassamble, tcp_reassamble);
         return (0);
+    }
     q = LIST_FIRST(&tp->t_segq);
     if (!q || q->tqe_th->th_seq != tp->rcv_nxt)
+    {
+        STAM_PROFILE_STOP(&pData->StatTCP_reassamble, tcp_reassamble);
         return (0);
+    }
     do
     {
         tp->rcv_nxt += q->tqe_len;
@@ -237,7 +246,7 @@ present:
             if (so->so_emu)
             {
                 if (tcp_emu(pData, so, q->tqe_m))
-                    sbappend(pData, so, q->tqe_m); 
+                    sbappend(pData, so, q->tqe_m);
             }
             else
                 sbappend(pData, so, q->tqe_m);
@@ -249,6 +258,7 @@ present:
     }
     while (q && q->tqe_th->th_seq == tp->rcv_nxt);
 
+    STAM_PROFILE_STOP(&pData->StatTCP_reassamble, tcp_reassamble);
     return flags;
 }
 
@@ -272,13 +282,14 @@ tcp_input(PNATState pData, register struct mbuf *m, int iphlen, struct socket *i
     int iss = 0;
     u_long tiwin;
 /*  int ts_present = 0; */
+    STAM_PROFILE_START(&pData->StatTCP_input, counter_input);
 
     DEBUG_CALL("tcp_input");
     DEBUG_ARGS((dfd," m = %8lx  iphlen = %2d  inso = %lx\n",
                 (long )m, iphlen, (long )inso ));
-    
-    if (inso != NULL) 
-    { 
+
+    if (inso != NULL)
+    {
         QSOCKET_LOCK(tcb);
         SOCKET_LOCK(inso);
         QSOCKET_UNLOCK(tcb);
@@ -289,20 +300,24 @@ tcp_input(PNATState pData, register struct mbuf *m, int iphlen, struct socket *i
     if (m == NULL)
     {
         so = inso;
-
+        Log4(("NAT: tcp_input: %R[natsock]\n", so));
         /* Re-set a few variables */
         tp = sototcpcb(so);
         m = so->so_m;
+
         so->so_m = 0;
         ti = so->so_ti;
-		/* @todo (r -vvl) clarify why it might happens */
+
+        /* @todo (r -vvl) clarify why it might happens */
         if (ti == NULL)
         {
             LogRel(("NAT: ti is null. can't do any reseting connection actions\n"));
             /* mbuf should be cleared in sofree called from tcp_close */
             tcp_close(pData, tp);
+            STAM_PROFILE_STOP(&pData->StatTCP_input, counter_input);
             return;
         }
+
         tiwin = ti->ti_win;
         tiflags = ti->ti_flags;
 
@@ -344,7 +359,6 @@ tcp_input(PNATState pData, register struct mbuf *m, int iphlen, struct socket *i
     if (cksum(m, len))
     {
         tcpstat.tcps_rcvbadsum++;
-        Log2(("checksum is invalid => drop\n"));
         goto drop;
     }
 
@@ -357,7 +371,6 @@ tcp_input(PNATState pData, register struct mbuf *m, int iphlen, struct socket *i
         || off > tlen)
     {
         tcpstat.tcps_rcvbadoff++;
-        Log2(("ti_off(tlen(%d)<%d<(tcphdr(%d))) is invalid =>drop\n", tlen, off, sizeof(struct tcphdr)));
         goto drop;
     }
     tlen -= off;
@@ -419,27 +432,25 @@ findso:
     {
         struct socket *sonxt;
         QSOCKET_UNLOCK(tcb);
-        /* @todo fix SOLOOKUP macrodefinition to be usable here */ 
+        /* @todo fix SOLOOKUP macrodefinition to be usable here */
 #ifndef VBOX_WITH_SLIRP_MT
         so = solookup(&tcb, ti->ti_src, ti->ti_sport,
                       ti->ti_dst, ti->ti_dport);
 #else
         so = NULL;
-        QSOCKET_FOREACH(so, sonxt, tcp)        
+        QSOCKET_FOREACH(so, sonxt, tcp)
         /* { */
             if (   so->so_lport        == ti->ti_sport
                 && so->so_laddr.s_addr == ti->ti_src.s_addr
                 && so->so_faddr.s_addr == ti->ti_dst.s_addr
                 && so->so_fport        == ti->ti_dport
-                && so->so_deleted != 1) 
+                && so->so_deleted != 1)
             {
-                Log2(("lock: %s:%d We found socket %R[natsock]\n", __FUNCTION__, __LINE__, so));
                 break; /* so is locked here */
             }
         LOOP_LABEL(tcp, so, sonxt);
         }
         if (so == &tcb) {
-            Log2(("lock: %s:%d Haven't find anything \n", __FUNCTION__, __LINE__));
             so = NULL;
         }
 #endif
@@ -448,7 +459,7 @@ findso:
             tcp_last_so = so;
         }
         ++tcpstat.tcps_socachemiss;
-    } 
+    }
     else
     {
         SOCKET_LOCK(so);
@@ -468,7 +479,6 @@ findso:
      * the only flag set, then create a session, mark it
      * as if it was LISTENING, and continue...
      */
-    Log2(("so = %R[natsock]\n", so));
     if (so == 0)
     {
         if ((tiflags & (TH_SYN|TH_FIN|TH_RST|TH_URG|TH_ACK)) != TH_SYN)
@@ -481,10 +491,9 @@ findso:
             RTMemFree(so); /* Not sofree (if it failed, it's not insqued) */
             goto dropwithreset;
         }
-        
-        SOCKET_LOCK(so); 
-        sbreserve(&so->so_snd, tcp_sndspace);
-        sbreserve(&so->so_rcv, tcp_rcvspace);
+        SOCKET_LOCK(so);
+        sbreserve(pData, &so->so_snd, tcp_sndspace);
+        sbreserve(pData, &so->so_rcv, tcp_rcvspace);
 
 /*      tcp_last_so = so; */  /* XXX ? */
 /*      tp = sototcpcb(so);    */
@@ -506,9 +515,8 @@ findso:
      * a retransmit of the SYN.  Whether it's a retransmit SYN
      * or something else, we nuke it.
      */
-    if (so->so_state & SS_ISFCONNECTING) 
+    if (so->so_state & SS_ISFCONNECTING)
     {
-        Log2(("so_state(%x) of %R[natsock] is still connecting =>drop\n", so->so_state, so));
         goto drop;
     }
 
@@ -519,7 +527,6 @@ findso:
         goto dropwithreset;
     if (tp->t_state == TCPS_CLOSED)
     {
-        Log2(("t_state(%x) is closed =>drop\n", tp->t_state));
         goto drop;
     }
 
@@ -643,6 +650,7 @@ findso:
                   (void) tcp_output(pData, tp);
 
               SOCKET_UNLOCK(so);
+              STAM_PROFILE_STOP(&pData->StatTCP_input, counter_input);
               return;
             }
         }
@@ -688,6 +696,7 @@ findso:
             tp->t_flags |= TF_ACKNOW;
             tcp_output(pData, tp);
             SOCKET_UNLOCK(so);
+            STAM_PROFILE_STOP(&pData->StatTCP_input, counter_input);
             return;
         }
     } /* header prediction */
@@ -723,14 +732,12 @@ findso:
         case TCPS_LISTEN:
         {
             if (tiflags & TH_RST) {
-                Log2(("RST(%x) is on listen =>drop\n", tiflags));
                 goto drop;
             }
             if (tiflags & TH_ACK)
                 goto dropwithreset;
-            if ((tiflags & TH_SYN) == 0) 
+            if ((tiflags & TH_SYN) == 0)
             {
-                Log2(("SYN(%x) is off on listen =>drop\n", tiflags));
                 goto drop;
             }
 
@@ -789,6 +796,7 @@ findso:
                 tp->t_state = TCPS_SYN_RECEIVED;
             }
             SOCKET_UNLOCK(so);
+            STAM_PROFILE_STOP(&pData->StatTCP_input, counter_input);
             return;
 
 cont_conn:
@@ -818,7 +826,6 @@ cont_input:
             tp->t_state = TCPS_SYN_RECEIVED;
             tp->t_timer[TCPT_KEEP] = TCPTV_KEEP_INIT;
             tcpstat.tcps_accepts++;
-            Log2(("hit trimthenstep6\n"));
             goto trimthenstep6;
         } /* case TCPS_LISTEN */
 
@@ -844,13 +851,11 @@ cont_input:
             {
                 if (tiflags & TH_ACK)
                     tp = tcp_drop(pData, tp, 0); /* XXX Check t_softerror! */
-                Log2(("RST(%x) is on SYN_SENT =>drop\n", tiflags));
                 goto drop;
             }
 
-            if ((tiflags & TH_SYN) == 0) 
+            if ((tiflags & TH_SYN) == 0)
             {
-                Log2(("SYN(%x) bit is off on SYN_SENT =>drop\n", tiflags));
                 goto drop;
             }
             if (tiflags & TH_ACK)
@@ -1072,7 +1077,7 @@ trimthenstep6:
 #if 0
     if (   ts_present
         && SEQ_LEQ(ti->ti_seq, tp->last_ack_sent)
-        && SEQ_LT(tp->last_ack_sent, ti->ti_seq + ti->ti_len + ((tiflags & (TH_SYN|TH_FIN)) != 0))) 
+        && SEQ_LT(tp->last_ack_sent, ti->ti_seq + ti->ti_len + ((tiflags & (TH_SYN|TH_FIN)) != 0)))
     {
         tp->ts_recent_age = tcp_now;
         tp->ts_recent = ts_val;
@@ -1102,7 +1107,6 @@ trimthenstep6:
             case TCPS_CLOSE_WAIT:
 /*              so->so_error = ECONNRESET; */
 close:
-                Log2(("closing...=>drop\n", tp->t_state)); 
                 tp->t_state = TCPS_CLOSED;
                 tcpstat.tcps_drops++;
                 tp = tcp_close(pData, tp);
@@ -1111,7 +1115,6 @@ close:
             case TCPS_CLOSING:
             case TCPS_LAST_ACK:
             case TCPS_TIME_WAIT:
-                Log2(("t_state is (%x) sort of close =>drop\n", tp->t_state)); 
                 tp = tcp_close(pData, tp);
                 goto drop;
         }
@@ -1129,9 +1132,8 @@ close:
     /*
      * If the ACK bit is off we drop the segment and return.
      */
-    if ((tiflags & TH_ACK) == 0) 
+    if ((tiflags & TH_ACK) == 0)
     {
-        Log2(("ACK(%x) bit is off =>drop\n", tiflags)); 
         goto drop;
     }
 
@@ -1173,7 +1175,6 @@ close:
             (void) tcp_reass(pData, tp, (struct tcphdr *)0, (int *)0, (struct mbuf *)0);
             tp->snd_wl1 = ti->ti_seq - 1;
             /* Avoid ack processing; snd_una==ti_ack  =>  dup ack */
-            Log2(("hit synrx_to_est\n"));
             goto synrx_to_est;
             /* fall into ... */
 
@@ -1242,14 +1243,12 @@ close:
                             tp->t_maxseg * tp->t_dupacks;
                         if (SEQ_GT(onxt, tp->snd_nxt))
                             tp->snd_nxt = onxt;
-                        Log2(("t_dupacks(%d) == tcprexmtthresh(%d)=>drop\n", tp->t_dupacks, tcprexmtthresh));
                         goto drop;
                     }
                     else if (tp->t_dupacks > tcprexmtthresh)
                     {
                         tp->snd_cwnd += tp->t_maxseg;
                         (void) tcp_output(pData, tp);
-                        Log2(("t_dupacks(%d) > tcprexmtthresh(%d)=>drop\n", tp->t_dupacks, tcprexmtthresh));
                         goto drop;
                     }
                 }
@@ -1258,7 +1257,6 @@ close:
                 break;
             }
 synrx_to_est:
-            Log2(("enter synrx_to_est\n"));
             /*
              * If the congestion window was inflated to account
              * for the other side's cached packets, retract it.
@@ -1396,7 +1394,6 @@ synrx_to_est:
                 case TCPS_LAST_ACK:
                     if (ourfinisacked)
                     {
-                        Log2(("ourfinisacked=>drop\n"));
                         tp = tcp_close(pData, tp);
                         goto drop;
                     }
@@ -1487,7 +1484,6 @@ step6:
         if (SEQ_GT(tp->rcv_nxt, tp->rcv_up))
             tp->rcv_up = tp->rcv_nxt;
 dodata:
-    Log2(("do data hit!\n"));
 
     /*
      * If this is a small packet, then ACK now - with Nagel
@@ -1525,9 +1521,9 @@ dodata:
             tcpstat.tcps_rcvbyte += tlen;
             if (so->so_state & SS_FCANTRCVMORE)
                 m_freem(pData, m);
-            else 
+            else
             {
-                if (so->so_emu) 
+                if (so->so_emu)
                 {
                     if (tcp_emu(pData, so, m))
                         sbappend(pData, so, m);
@@ -1627,6 +1623,7 @@ dodata:
         tcp_output(pData, tp);
 
     SOCKET_UNLOCK(so);
+    STAM_PROFILE_STOP(&pData->StatTCP_input, counter_input);
     return;
 
 dropafterack:
@@ -1641,6 +1638,7 @@ dropafterack:
     tp->t_flags |= TF_ACKNOW;
     (void) tcp_output(pData, tp);
     SOCKET_UNLOCK(so);
+    STAM_PROFILE_STOP(&pData->StatTCP_input, counter_input);
     return;
 
 dropwithreset:
@@ -1656,6 +1654,7 @@ dropwithreset:
 
     if (so != &tcb)
         SOCKET_UNLOCK(so);
+    STAM_PROFILE_STOP(&pData->StatTCP_input, counter_input);
     return;
 
 drop:
@@ -1665,12 +1664,13 @@ drop:
     m_free(pData, m);
 
 #ifdef VBOX_WITH_SLIRP_MT
-    if (RTCritSectIsOwned(&so->so_mutex)) 
+    if (RTCritSectIsOwned(&so->so_mutex))
     {
         SOCKET_UNLOCK(so);
     }
 #endif
 
+    STAM_PROFILE_STOP(&pData->StatTCP_input, counter_input);
     return;
 }
 
@@ -1897,8 +1897,8 @@ tcp_mss(PNATState pData, register struct tcpcb *tp, u_int offer)
 
     tp->snd_cwnd = mss;
 
-    sbreserve(&so->so_snd, tcp_sndspace+((tcp_sndspace%mss)?(mss-(tcp_sndspace%mss)):0));
-    sbreserve(&so->so_rcv, tcp_rcvspace+((tcp_rcvspace%mss)?(mss-(tcp_rcvspace%mss)):0));
+    sbreserve(pData, &so->so_snd, tcp_sndspace+((tcp_sndspace%mss)?(mss-(tcp_sndspace%mss)):0));
+    sbreserve(pData, &so->so_rcv, tcp_rcvspace+((tcp_rcvspace%mss)?(mss-(tcp_rcvspace%mss)):0));
 
     DEBUG_MISC((dfd, " returning mss = %d\n", mss));
 
