@@ -77,7 +77,9 @@ udp_input(PNATState pData, register struct mbuf *m, int iphlen)
 
     DEBUG_CALL("udp_input");
     DEBUG_ARG("m = %lx", (long)m);
+    ip = mtod(m, struct ip *);
     DEBUG_ARG("iphlen = %d", iphlen);
+    Log2(("%R[IP4] iphlen = %d\n", &ip->ip_dst, iphlen));
 
     udpstat.udps_ipackets++;
 
@@ -228,14 +230,15 @@ udp_input(PNATState pData, register struct mbuf *m, int iphlen)
     so->so_faddr = ip->ip_dst;   /* XXX */
     so->so_fport = uh->uh_dport; /* XXX */
 
-#ifdef VBOX_WITH_SLIRP_DNS_PROXY
+    /*
+     * DNS proxy
+     */
     if (   (ip->ip_dst.s_addr == htonl(ntohl(special_addr.s_addr) | CTL_DNS))
         && (ntohs(uh->uh_dport) == 53)) 
     {
         dnsproxy_query(pData, so, m, iphlen);
         goto bad; /* it isn't bad, probably better to add additional label done for boot/tftf :)  */
     }
-#endif
 
     iphlen += sizeof(struct udphdr);
     m->m_len -= iphlen;
@@ -258,7 +261,8 @@ udp_input(PNATState pData, register struct mbuf *m, int iphlen)
         m->m_len += iphlen;
         m->m_data -= iphlen;
         *ip = save_ip;
-        DEBUG_MISC((dfd,"udp tx errno = %d-%s\n", errno, strerror(errno)));
+        DEBUG_MISC((dfd,"NAT: UDP tx errno = %d-%s (on sent to %R[IP4])\n", errno, 
+                strerror(errno), &ip->ip_dst));
         icmp_error(pData, m, ICMP_UNREACH, ICMP_UNREACH_NET, 0, strerror(errno));
         /* in case we receive ICMP on this socket we'll aware that ICMP has been already sent to host*/
         so->so_m = NULL; 
@@ -275,6 +279,8 @@ udp_input(PNATState pData, register struct mbuf *m, int iphlen)
     return;
 
 bad:
+    Log2(("NAT: UDP datagram to %R[IP4] with size(%d) claimed as bad\n", 
+        &ip->ip_dst, ip->ip_len));
     m_freem(pData, m);
     return;
 }
@@ -373,9 +379,13 @@ udp_attach(PNATState pData, struct socket *so, int service_port)
          * (sendto() on an unbound socket will bind it), it's done
          * here so that emulation of ytalk etc. don't have to do it
          */
+        memset(&addr, 0, sizeof(addr));
+#ifdef RT_OS_DARWIN
+        addr.sin_len = sizeof(addr);
+#endif
         addr.sin_family = AF_INET;
         addr.sin_port = service_port;
-        addr.sin_addr.s_addr = INADDR_ANY;
+        addr.sin_addr.s_addr = pData->bindIP.s_addr;
         fd_nonblock(so->s);
         if (bind(so->s, (struct sockaddr *)&addr, sizeof(addr)) < 0)
         {
@@ -465,6 +475,7 @@ udp_tos(struct socket *so)
 void
 udp_emu(PNATState pData, struct socket *so, struct mbuf *m)
 {
+#ifndef VBOX_WITH_SLIRP_ALIAS
     struct sockaddr_in addr;
     socklen_t addrlen = sizeof(addr);
 #ifdef EMULATE_TALK
@@ -582,7 +593,7 @@ udp_emu(PNATState pData, struct socket *so, struct mbuf *m)
                 int s;
                 u_short temp_port;
 
-                for(req = req_tbl; req; req = req->next)
+                for (req = req_tbl; req; req = req->next)
                     if (so == req->udp_so)
                         break;          /* found it */
 
@@ -704,10 +715,13 @@ udp_emu(PNATState pData, struct socket *so, struct mbuf *m)
             }
             return;
     }
+#else /*!VBOX_WITH_SLIRP_ALIAS*/
+    so->so_emu = 0;
+#endif /* VBOX_WITH_SLIRP_ALIAS */
 }
 
 struct socket *
-udp_listen(PNATState pData, u_int port, u_int32_t laddr, u_int lport, int flags)
+udp_listen(PNATState pData, u_int32_t bind_addr, u_int port, u_int32_t laddr, u_int lport, int flags)
 {
     struct sockaddr_in addr;
     struct socket *so;
@@ -732,12 +746,17 @@ udp_listen(PNATState pData, u_int port, u_int32_t laddr, u_int lport, int flags)
     NSOCK_INC();
     QSOCKET_UNLOCK(udb);
 
+    memset(&addr, 0, sizeof(addr));
+#ifdef RT_OS_DARWIN
+    addr.sin_len = sizeof(addr);
+#endif
     addr.sin_family = AF_INET;
-    addr.sin_addr.s_addr = INADDR_ANY;
+    addr.sin_addr.s_addr = bind_addr;
     addr.sin_port = port;
 
     if (bind(so->s,(struct sockaddr *)&addr, addrlen) < 0)
     {
+        LogRel(("NAT: bind to %R[IP4] has been failed\n", &addr.sin_addr));
         udp_detach(pData, so);
         return NULL;
     }
