@@ -25,7 +25,7 @@
 #include <VBox/com/com.h>
 #include <VBox/com/string.h>
 #include <VBox/com/ErrorInfo.h>
-#include <VBox/com/errorprint2.h>
+#include <VBox/com/errorprint.h>
 
 #include <VBox/com/VirtualBox.h>
 
@@ -46,7 +46,7 @@ int handleSnapshot(HandlerArg *a)
     /* the first argument must be the VM */
     ComPtr<IMachine> machine;
     /* assume it's a UUID */
-    rc = a->virtualBox->GetMachine(Guid(a->argv[0]), machine.asOutParam());
+    rc = a->virtualBox->GetMachine(Bstr(a->argv[0]), machine.asOutParam());
     if (FAILED(rc) || !machine)
     {
         /* must be a name */
@@ -54,7 +54,7 @@ int handleSnapshot(HandlerArg *a)
     }
     if (!machine)
         return 1;
-    Guid guid;
+    Bstr guid;
     machine->COMGETTER(Id)(guid.asOutParam());
 
     do
@@ -93,7 +93,9 @@ int handleSnapshot(HandlerArg *a)
             CHECK_ERROR_BREAK(console, TakeSnapshot(name, desc, progress.asOutParam()));
 
             showProgress(progress);
-            progress->COMGETTER(ResultCode)(&rc);
+            LONG iRc;
+            progress->COMGETTER(ResultCode)(&iRc);
+            rc = iRc;
             if (FAILED(rc))
             {
                 com::ProgressErrorInfo info(progress);
@@ -116,7 +118,7 @@ int handleSnapshot(HandlerArg *a)
             ComPtr<ISnapshot> snapshot;
 
             /* assume it's a UUID */
-            Guid guid(a->argv[2]);
+            Bstr guid(a->argv[2]);
             if (!guid.isEmpty())
             {
                 CHECK_ERROR_BREAK(machine, GetSnapshot(guid, snapshot.asOutParam()));
@@ -133,7 +135,9 @@ int handleSnapshot(HandlerArg *a)
             CHECK_ERROR_BREAK(console, DiscardSnapshot(guid, progress.asOutParam()));
 
             showProgress(progress);
-            progress->COMGETTER(ResultCode)(&rc);
+            LONG iRc;
+            progress->COMGETTER(ResultCode)(&iRc);
+            rc = iRc;
             if (FAILED(rc))
             {
                 com::ProgressErrorInfo info(progress);
@@ -172,7 +176,9 @@ int handleSnapshot(HandlerArg *a)
             }
 
             showProgress(progress);
-            progress->COMGETTER(ResultCode)(&rc);
+            LONG iRc;
+            progress->COMGETTER(ResultCode)(&iRc);
+            rc = iRc;
             if (FAILED(rc))
             {
                 com::ProgressErrorInfo info(progress);
@@ -202,7 +208,7 @@ int handleSnapshot(HandlerArg *a)
             else
             {
                 /* assume it's a UUID */
-                Guid guid(a->argv[2]);
+                Bstr guid(a->argv[2]);
                 if (!guid.isEmpty())
                 {
                     CHECK_ERROR_BREAK(machine, GetSnapshot(guid, snapshot.asOutParam()));
@@ -265,7 +271,7 @@ int handleSnapshot(HandlerArg *a)
             ComPtr<ISnapshot> snapshot;
 
             /* assume it's a UUID */
-            Guid guid(a->argv[2]);
+            Bstr guid(a->argv[2]);
             if (!guid.isEmpty())
             {
                 CHECK_ERROR_BREAK(machine, GetSnapshot(guid, snapshot.asOutParam()));
