@@ -36,6 +36,7 @@
 #include <QColor>
 #include <QDialog>
 #include <QMenu>
+#include <QPointer>
 
 #ifdef VBOX_WITH_DEBUGGER_GUI
 # include <VBox/dbggui.h>
@@ -57,6 +58,8 @@ class VBoxSwitchMenu;
 
 class VBoxChangeDockIconUpdateEvent;
 
+class VBoxMiniToolBar;
+
 /* We want to make the first action highlighted but not
  * selected, but Qt makes the both or neither one of this,
  * so, just move the focus to the next eligible object,
@@ -249,6 +252,10 @@ private slots:
     void switchToFullscreen (bool aOn, bool aSeamless);
     void setViewInSeamlessMode (const QRect &aTargetRect);
 
+    void mtExitMode();
+    void mtCloseVM();
+    void mtMaskUpdate();
+
 private:
 
     /** Popup version of the main menu */
@@ -306,6 +313,7 @@ private:
     QAction *mDevicesSFMenuSeparator;
 
     QMenu *mVMMenu;
+    QMenu *mMiniVMMenu;
     QMenu *mDevicesMenu;
     QMenu *mDevicesMountFloppyMenu;
     QMenu *mDevicesMountDVDMenu;
@@ -369,7 +377,7 @@ private:
     bool mIsGraphicsSupported : 1;
     bool mIsWaitingModeResize : 1;
     bool was_max : 1;
-    QObjectList hidden_children;
+    QList < QPointer <QWidget> > mHiddenChildren;
     int console_style;
     QPalette mErasePalette;
 
@@ -391,6 +399,8 @@ private:
     /* For the fade effect if the the window goes fullscreen */
     CGDisplayFadeReservationToken mFadeToken;
 #endif
+
+    VBoxMiniToolBar *mMiniToolBar;
 };
 
 
