@@ -248,16 +248,11 @@ bool VBoxProblemReporter::showModalProgressDialog (
     CProgress &aProgress, const QString &aTitle, QWidget *aParent,
     int aMinDuration)
 {
-    QApplication::setOverrideCursor (QCursor (Qt::WaitCursor));
-
-    VBoxProgressDialog progressDlg (aProgress, aTitle, aMinDuration,
-                                    aParent);
+    VBoxProgressDialog progressDlg (aProgress, aTitle, aMinDuration, aParent);
 
     /* run the dialog with the 100 ms refresh interval */
     progressDlg.run (100);
 
-    QApplication::restoreOverrideCursor();
-
     return true;
 }
 
@@ -296,18 +291,34 @@ QWidget *VBoxProblemReporter::mainWindowShown() const
 // Generic Problem handlers
 /////////////////////////////////////////////////////////////////////////////
 
-bool VBoxProblemReporter::askForOverridingFileIfExists (const QString& aPath, QWidget *aParent /* = NULL */) const
+bool VBoxProblemReporter::askForOverridingFile (const QString& aPath, QWidget *aParent /* = NULL */) const
 {
-    QFileInfo fi (aPath);
-    if (fi.exists())
-        return messageYesNo (aParent, Question, tr ("A file named <b>%1</b> already exists. Are you sure you want to replace it?<br /><br />The file already exists in \"%2\". Replacing it will overwrite its contents.").arg (fi.fileName()). arg (fi.absolutePath()));
+    return messageYesNo (aParent, Question, tr ("A file named <b>%1</b> already exists. Are you sure you want to replace it?<br /><br />Replacing it will overwrite its contents.").arg (aPath));
+}
+
+bool VBoxProblemReporter::askForOverridingFiles (const QVector<QString>& aPaths, QWidget *aParent /* = NULL */) const
+{
+    if (aPaths.size() == 1)
+        /* If it is only one file use the single question versions above */
+        return askForOverridingFile (aPaths.at (0), aParent);
+    else if (aPaths.size() > 1)
+        return messageYesNo (aParent, Question, tr ("The following files already exist:<br /><br />%1<br /><br />Are you sure you want to replace them? Replacing them will overwrite their contents.").arg (QStringList(aPaths.toList()).join ("<br />")));
     else
         return true;
 }
 
-bool VBoxProblemReporter::askForOverridingFilesIfExists (const QStringList& aPaths, QWidget *aParent /* = NULL */) const
+bool VBoxProblemReporter::askForOverridingFileIfExists (const QString& aPath, QWidget *aParent /* = NULL */) const
 {
-    QStringList existingFiles;
+    QFileInfo fi (aPath);
+    if (fi.exists())
+        return askForOverridingFile (aPath, aParent);
+    else
+        return true;
+}
+
+bool VBoxProblemReporter::askForOverridingFilesIfExists (const QVector<QString>& aPaths, QWidget *aParent /* = NULL */) const
+{
+    QVector<QString> existingFiles;
     foreach (const QString &file, aPaths)
     {
         QFileInfo fi (file);
@@ -318,7 +329,7 @@ bool VBoxProblemReporter::askForOverridingFilesIfExists (const QStringList& aPat
         /* If it is only one file use the single question versions above */
         return askForOverridingFileIfExists (existingFiles.at (0), aParent);
     else if (existingFiles.size() > 1)
-        return messageYesNo (aParent, Question, tr ("The following files already exist:<br /><br />%1<br /><br />Are you sure you want to replace them? Replacing them will overwrite their contents.").arg (existingFiles.join ("<br />")));
+        return askForOverridingFiles (existingFiles, aParent);
     else
         return true;
 }
@@ -1190,6 +1201,14 @@ void VBoxProblemReporter::cannotCloseMedium (
         formatErrorInfo (aResult));
 }
 
+void VBoxProblemReporter::cannotEjectDrive()
+{
+    message (mainWindowShown(), Warning,
+             tr ("Failed to eject the disk from the virtual drive. "
+                 "The drive may be locked by the guest operating system. "
+                 "Please check this and try again."));
+}
+
 void VBoxProblemReporter::cannotOpenSession (const CSession &session)
 {
     Assert (session.isNull());
@@ -1231,8 +1250,6 @@ void VBoxProblemReporter::cannotGetMediaAccessibility (const VBoxMedium &aMedium
         formatErrorInfo (aMedium.result()));
 }
 
-#if defined Q_WS_WIN
-
 int VBoxProblemReporter::confirmDeletingHostInterface (const QString &aName,
                                                        QWidget *aParent)
 {
@@ -1284,8 +1301,6 @@ void VBoxProblemReporter::cannotRemoveHostInterface (
         formatErrorInfo (progress.GetErrorInfo()));
 }
 
-#endif
-
 void VBoxProblemReporter::cannotAttachUSBDevice (const CConsole &console,
                                                  const QString &device)
 {
@@ -1528,15 +1543,27 @@ void VBoxProblemReporter::cannotConnectRegister (QWidget *aParent,
 void VBoxProblemReporter::showRegisterResult (QWidget *aParent,
                                               const QString &aResult)
 {
-    aResult == "OK" ?
+    if (aResult == "OK")
+    {
+        /* On successful registration attempt */
         message (aParent, Info,
                  tr ("<p>Congratulations! You have been successfully registered "
                      "as a user of VirtualBox.</p>"
                      "<p>Thank you for finding time to fill out the "
-                     "registration form!</p>")) :
+                     "registration form!</p>"));
+    }
+    else
+    {
+        QString parsed;
+
+        /* Else parse and translate special key-words */
+        if (aResult == "AUTHFAILED")
+            parsed = tr ("<p>Invalid e-mail address or password specified.</p>");
+
         message (aParent, Error,
-                 tr ("<p>Failed to register the VirtualBox product</p><p>%1</p>")
-                 .arg (aResult));
+                 tr ("<p>Failed to register the VirtualBox product.</p><p>%1</p>")
+                 .arg (parsed.isNull() ? aResult : parsed));
+    }
 }
 
 void VBoxProblemReporter::showUpdateSuccess (QWidget *aParent,
@@ -2016,6 +2043,26 @@ void VBoxProblemReporter::cannotImportAppliance (const CProgress &aProgress, CAp
              formatErrorInfo (aProgress.GetErrorInfo()));
 }
 
+void VBoxProblemReporter::cannotCheckFiles (const CProgress &aProgress, QWidget *aParent /* = NULL */) const
+{
+    AssertWrapperOk (aProgress);
+
+    message (aParent ? aParent : mainWindowShown(),
+             Error,
+             tr ("Failed to check files."),
+             formatErrorInfo (aProgress.GetErrorInfo()));
+}
+
+void VBoxProblemReporter::cannotRemoveFiles (const CProgress &aProgress, QWidget *aParent /* = NULL */) const
+{
+    AssertWrapperOk (aProgress);
+
+    message (aParent ? aParent : mainWindowShown(),
+             Error,
+             tr ("Failed to remove file."),
+             formatErrorInfo (aProgress.GetErrorInfo()));
+}
+
 void VBoxProblemReporter::cannotExportAppliance (CAppliance *aAppliance, QWidget *aParent /* = NULL */) const
 {
     if (aAppliance->isNull())
