@@ -5,15 +5,73 @@
  */
 
 /*
- * Copyright (C) 2006-2007 innotek GmbH
+ * Copyright (C) 2006-2007 Sun Microsystems, Inc.
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
  * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License as published by the Free Software Foundation,
- * in version 2 as it comes in the "COPYING" file of the VirtualBox OSE
- * distribution. VirtualBox OSE is distributed in the hope that it will
- * be useful, but WITHOUT ANY WARRANTY of any kind.
+ * General Public License (GPL) as published by the Free Software
+ * Foundation, in version 2 as it comes in the "COPYING" file of the
+ * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
+ * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ *
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
+ * Clara, CA 95054 USA or visit http://www.sun.com if you need
+ * additional information or have any questions.
+ */
+
+
+/** @page pg_hostclip       The Shared Clipboard Host Service
+ *
+ * The shared clipboard host service provides a proxy between the host's
+ * clipboard and a similar proxy running on a guest.  The service is split
+ * into a platform-independant core and platform-specific backends.  The
+ * service defines two communication protocols - one to communicate with the
+ * clipboard service running on the guest, and one to communicate with the
+ * backend.  These will be described in a very skeletal fashion here.
+ *
+ * @section sec_hostclip_guest_proto  The guest communication protocol
+ *
+ * The guest clipboard service communicates with the host service via HGCM
+ * (the host service runs as an HGCM service).  The guest clipboard must
+ * connect to the host service before all else (Windows hosts currently only
+ * support one simultaneous connection).  Once it has connected, it can send
+ * HGCM messages to the host services, some of which will receive replies from
+ * the host.  The host can only reply to a guest message, it cannot initiate
+ * any communication.  The guest can in theory send any number of messages in
+ * parallel (see the descriptions of the messages for the practice), and the
+ * host will receive these in sequence, and may reply to them at once
+ * (releasing the caller in the guest) or defer the reply until later.
+ *
+ * There are currently four messages defined.  The first is
+ * VBOX_SHARED_CLIPBOARD_FN_GET_HOST_MSG, which waits for a message from the
+ * host.  Host messages currently defined are
+ * VBOX_SHARED_CLIPBOARD_HOST_MSG_QUIT (unused),
+ * VBOX_SHARED_CLIPBOARD_HOST_MSG_READ_DATA (request that the guest send the
+ * contents of its clipboard to the host) and
+ * VBOX_SHARED_CLIPBOARD_HOST_MSG_FORMATS (to notify the guest that new
+ * clipboard data is available).  If a host message is sent while the guest is
+ * not waiting, it will be queued until the guest requests it.  At most one
+ * host message of each type will be kept in the queue.  The host code only
+ * supports a single simultaneous VBOX_SHARED_CLIPBOARD_FN_GET_HOST_MSG call
+ * from the guest.
+ *
+ * The second guest message is VBOX_SHARED_CLIPBOARD_FN_FORMATS, which tells
+ * the host that the guest has new clipboard data available.  The third is
+ * VBOX_SHARED_CLIPBOARD_FN_READ_DATA, which asks the host to send its
+ * clipboard data and waits until it arrives.  The host supports at most one
+ * simultaneous VBOX_SHARED_CLIPBOARD_FN_READ_DATA call from the guest - if a
+ * second call is made before the first has returned, the first will be
+ * aborted.
+ *
+ * The last guest message is VBOX_SHARED_CLIPBOARD_FN_WRITE_DATA, which is
+ * used to send the contents of the guest clipboard to the host.  This call
+ * should be used after the host has requested data from the guest.
+ *
+ * @section sec_hostclip_backend_proto  The communication protocol with the
+ *                                      platform-specific backend
+ *
+ * This section may be written in the future :)
  */
 
 #include <VBox/HostServices/VBoxClipboardSvc.h>
@@ -27,8 +85,6 @@
 
 #include "VBoxClipboard.h"
 
-#define VBOX_HGCM_PARM_COUNT(a) ((sizeof (a) - sizeof (((a *)0)->hdr)) / sizeof (HGCMFunctionParameter))
-
 static void VBoxHGCMParmUInt32Set (VBOXHGCMSVCPARM *pParm, uint32_t u32)
 {
     pParm->type = VBOX_HGCM_SVC_PARM_32BIT;
@@ -77,6 +133,11 @@ static void *g_pvExtension;
 
 static VBOXCLIPBOARDCLIENTDATA *g_pClient;
 
+/* Serialization of data reading and format announcements from the RDP client. */
+static bool g_fReadingData = false;
+static bool g_fDelayedAnnouncement = false;
+static uint32_t g_u32DelayedFormats = 0;
+
 static uint32_t vboxSvcClipboardMode (void)
 {
     return g_u32Mode;
@@ -100,7 +161,7 @@ static void vboxSvcClipboardModeSet (uint32_t u32Mode)
 
 bool vboxSvcClipboardLock (void)
 {
-    return VBOX_SUCCESS(RTCritSectEnter (&critsect));
+    return RT_SUCCESS(RTCritSectEnter (&critsect));
 }
 
 void vboxSvcClipboardUnlock (void)
@@ -108,7 +169,7 @@ void vboxSvcClipboardUnlock (void)
     RTCritSectLeave (&critsect);
 }
 
-/* Set the HGCM parameters according to pending messages. 
+/* Set the HGCM parameters according to pending messages.
  * Executed under the clipboard lock.
  */
 static bool vboxSvcClipboardReturnMsg (VBOXCLIPBOARDCLIENTDATA *pClient, VBOXHGCMSVCPARM paParms[])
@@ -116,21 +177,21 @@ static bool vboxSvcClipboardReturnMsg (VBOXCLIPBOARDCLIENTDATA *pClient, VBOXHGC
     /* Message priority is taken into account. */
     if (pClient->fMsgQuit)
     {
-        LogFlow(("vboxSvcClipboardReturnMsg: Quit\n"));
+        LogRelFlow(("vboxSvcClipboardReturnMsg: Quit\n"));
         VBoxHGCMParmUInt32Set (&paParms[0], VBOX_SHARED_CLIPBOARD_HOST_MSG_QUIT);
         VBoxHGCMParmUInt32Set (&paParms[1], 0);
         pClient->fMsgQuit = false;
     }
     else if (pClient->fMsgReadData)
     {
-        LogFlow(("vboxSvcClipboardReturnMsg: ReadData %02X\n", pClient->u32RequestedFormat));
+        LogRelFlow(("vboxSvcClipboardReturnMsg: ReadData %02X\n", pClient->u32RequestedFormat));
         VBoxHGCMParmUInt32Set (&paParms[0], VBOX_SHARED_CLIPBOARD_HOST_MSG_READ_DATA);
         VBoxHGCMParmUInt32Set (&paParms[1], pClient->u32RequestedFormat);
         pClient->fMsgReadData = false;
     }
     else if (pClient->fMsgFormats)
     {
-        LogFlow(("vboxSvcClipboardReturnMsg: Formats %02X\n", pClient->u32AvailableFormats));
+        LogRelFlow(("vboxSvcClipboardReturnMsg: Formats %02X\n", pClient->u32AvailableFormats));
         VBoxHGCMParmUInt32Set (&paParms[0], VBOX_SHARED_CLIPBOARD_HOST_MSG_FORMATS);
         VBoxHGCMParmUInt32Set (&paParms[1], pClient->u32AvailableFormats);
         pClient->fMsgFormats = false;
@@ -138,10 +199,10 @@ static bool vboxSvcClipboardReturnMsg (VBOXCLIPBOARDCLIENTDATA *pClient, VBOXHGC
     else
     {
         /* No pending messages. */
-        LogFlow(("vboxSvcClipboardReturnMsg: no message\n"));
+        LogRelFlow(("vboxSvcClipboardReturnMsg: no message\n"));
         return false;
     }
-    
+
     /* Message information assigned. */
     return true;
 }
@@ -154,7 +215,7 @@ void vboxSvcClipboardReportMsg (VBOXCLIPBOARDCLIENTDATA *pClient, uint32_t u32Ms
         {
             case VBOX_SHARED_CLIPBOARD_HOST_MSG_QUIT:
             {
-                LogFlow(("vboxSvcClipboardReportMsg: Quit\n"));
+                LogRelFlow(("vboxSvcClipboardReportMsg: Quit\n"));
                 pClient->fMsgQuit = true;
             } break;
             case VBOX_SHARED_CLIPBOARD_HOST_MSG_READ_DATA:
@@ -165,8 +226,8 @@ void vboxSvcClipboardReportMsg (VBOXCLIPBOARDCLIENTDATA *pClient, uint32_t u32Ms
                     /* Skip the message. */
                     break;
                 }
-                
-                LogFlow(("vboxSvcClipboardReportMsg: ReadData %02X\n", u32Formats));
+
+                LogRelFlow(("vboxSvcClipboardReportMsg: ReadData %02X\n", u32Formats));
                 pClient->u32RequestedFormat = u32Formats;
                 pClient->fMsgReadData = true;
             } break;
@@ -178,37 +239,37 @@ void vboxSvcClipboardReportMsg (VBOXCLIPBOARDCLIENTDATA *pClient, uint32_t u32Ms
                     /* Skip the message. */
                     break;
                 }
-                
-                LogFlow(("vboxSvcClipboardReportMsg: Formats %02X\n", u32Formats));
+
+                LogRelFlow(("vboxSvcClipboardReportMsg: Formats %02X\n", u32Formats));
                 pClient->u32AvailableFormats = u32Formats;
                 pClient->fMsgFormats = true;
             } break;
             default:
             {
                 /* Invalid message. */
-                LogFlow(("vboxSvcClipboardReportMsg: invalid message %d\n", u32Msg));
+                LogRelFlow(("vboxSvcClipboardReportMsg: invalid message %d\n", u32Msg));
             } break;
         }
-        
+
         if (pClient->fAsync)
         {
             /* The client waits for a responce. */
             bool fMessageReturned = vboxSvcClipboardReturnMsg (pClient, pClient->async.paParms);
-            
+
             /* Make a copy of the handle. */
             VBOXHGCMCALLHANDLE callHandle = pClient->async.callHandle;
-            
+
             if (fMessageReturned)
             {
                 /* There is a responce. */
                 pClient->fAsync = false;
             }
-            
+
             vboxSvcClipboardUnlock ();
-            
+
             if (fMessageReturned)
             {
-                LogFlow(("vboxSvcClipboardReportMsg: CallComplete\n"));
+                LogRelFlow(("vboxSvcClipboardReportMsg: CallComplete\n"));
                 g_pHelpers->pfnCallComplete (callHandle, VINF_SUCCESS);
             }
         }
@@ -221,42 +282,38 @@ void vboxSvcClipboardReportMsg (VBOXCLIPBOARDCLIENTDATA *pClient, uint32_t u32Ms
 
 static int svcInit (void)
 {
-    RTCritSectInit (&critsect);
-    vboxSvcClipboardModeSet (VBOX_SHARED_CLIPBOARD_MODE_OFF);
-    return vboxClipboardInit ();
+    int rc = RTCritSectInit (&critsect);
+
+    if (RT_SUCCESS (rc))
+    {
+        vboxSvcClipboardModeSet (VBOX_SHARED_CLIPBOARD_MODE_OFF);
+
+        rc = vboxClipboardInit ();
+
+        /* Clean up on failure, because 'svnUnload' will not be called
+         * if the 'svcInit' returns an error.
+         */
+        if (RT_FAILURE (rc))
+        {
+            RTCritSectDelete (&critsect);
+        }
+    }
+
+    return rc;
 }
 
-static DECLCALLBACK(int) svcUnload (void)
+static DECLCALLBACK(int) svcUnload (void *)
 {
     vboxClipboardDestroy ();
     RTCritSectDelete (&critsect);
     return VINF_SUCCESS;
 }
 
-static DECLCALLBACK(int) svcConnect (uint32_t u32ClientID, void *pvClient)
-{
-    VBOXCLIPBOARDCLIENTDATA *pClient = (VBOXCLIPBOARDCLIENTDATA *)pvClient;
-
-    /* Register the client. */
-    int rc = VINF_SUCCESS;
-
-    memset (pClient, 0, sizeof (*pClient));
-
-    pClient->u32ClientID = u32ClientID;
-
-    rc = vboxClipboardConnect (pClient);
-    
-    if (VBOX_SUCCESS (rc))
-    {
-        g_pClient = pClient;
-    }
-    
-    Log(("vboxClipboardConnect: rc = %Vrc\n", rc));
-
-    return rc;
-}
-
-static DECLCALLBACK(int) svcDisconnect (uint32_t u32ClientID, void *pvClient)
+/**
+ * Disconnect the host side of the shared clipboard and send a "host disconnected" message
+ * to the guest side.
+ */
+static DECLCALLBACK(int) svcDisconnect (void *, uint32_t u32ClientID, void *pvClient)
 {
     VBOXCLIPBOARDCLIENTDATA *pClient = (VBOXCLIPBOARDCLIENTDATA *)pvClient;
 
@@ -267,11 +324,45 @@ static DECLCALLBACK(int) svcDisconnect (uint32_t u32ClientID, void *pvClient)
     memset (pClient, 0, sizeof (*pClient));
 
     g_pClient = NULL;
-        
+
     return VINF_SUCCESS;
 }
 
-static DECLCALLBACK(void) svcCall (VBOXHGCMCALLHANDLE callHandle,
+static DECLCALLBACK(int) svcConnect (void *, uint32_t u32ClientID, void *pvClient)
+{
+    VBOXCLIPBOARDCLIENTDATA *pClient = (VBOXCLIPBOARDCLIENTDATA *)pvClient;
+
+    int rc = VINF_SUCCESS;
+
+    /* If there is already a client connected then we want to release it first. */
+    if (g_pClient != NULL)
+    {
+        uint32_t u32ClientID = g_pClient->u32ClientID;
+
+        svcDisconnect(NULL, u32ClientID, g_pClient);
+        /* And free the resources in the hgcm subsystem. */
+        g_pHelpers->pfnDisconnectClient(g_pHelpers->pvInstance, u32ClientID);
+    }
+
+    /* Register the client. */
+    memset (pClient, 0, sizeof (*pClient));
+
+    pClient->u32ClientID = u32ClientID;
+
+    rc = vboxClipboardConnect (pClient);
+
+    if (RT_SUCCESS (rc))
+    {
+        g_pClient = pClient;
+    }
+
+    LogRel2(("vboxClipboardConnect: rc = %Rrc\n", rc));
+
+    return rc;
+}
+
+static DECLCALLBACK(void) svcCall (void *,
+                                   VBOXHGCMCALLHANDLE callHandle,
                                    uint32_t u32ClientID,
                                    void *pvClient,
                                    uint32_t u32Function,
@@ -280,8 +371,8 @@ static DECLCALLBACK(void) svcCall (VBOXHGCMCALLHANDLE callHandle,
 {
     int rc = VINF_SUCCESS;
 
-    Log(("svcCall: u32ClientID = %d, fn = %d, cParms = %d, pparms = %d\n",
-         u32ClientID, u32Function, cParms, paParms));
+    LogRel2(("svcCall: u32ClientID = %d, fn = %d, cParms = %d, pparms = %d\n",
+             u32ClientID, u32Function, cParms, paParms));
 
     VBOXCLIPBOARDCLIENTDATA *pClient = (VBOXCLIPBOARDCLIENTDATA *)pvClient;
 
@@ -293,7 +384,7 @@ static DECLCALLBACK(void) svcCall (VBOXHGCMCALLHANDLE callHandle,
     for (i = 0; i < cParms; i++)
     {
         /** @todo parameters other than 32 bit */
-        Log(("    pparms[%d]: type %d value %d\n", i, paParms[i].type, paParms[i].u.uint32));
+        LogRel2(("    pparms[%d]: type %d value %d\n", i, paParms[i].type, paParms[i].u.uint32));
     }
 #endif
 
@@ -302,9 +393,9 @@ static DECLCALLBACK(void) svcCall (VBOXHGCMCALLHANDLE callHandle,
         case VBOX_SHARED_CLIPBOARD_FN_GET_HOST_MSG:
         {
             /* The quest requests a host message. */
-            Log(("svcCall: VBOX_SHARED_CLIPBOARD_FN_GET_HOST_MSG\n"));
+            LogRel2(("svcCall: VBOX_SHARED_CLIPBOARD_FN_GET_HOST_MSG\n"));
 
-            if (cParms != VBOX_HGCM_PARM_COUNT (VBoxClipboardGetHostMsg))
+            if (cParms != VBOX_SHARED_CLIPBOARD_CPARMS_GET_HOST_MSG)
             {
                 rc = VERR_INVALID_PARAMETER;
             }
@@ -320,7 +411,7 @@ static DECLCALLBACK(void) svcCall (VBOXHGCMCALLHANDLE callHandle,
                 if (vboxSvcClipboardLock ())
                 {
                     bool fMessageReturned = vboxSvcClipboardReturnMsg (pClient, paParms);
-                    
+
                     if (fMessageReturned)
                     {
                         /* Just return to the caller. */
@@ -335,7 +426,7 @@ static DECLCALLBACK(void) svcCall (VBOXHGCMCALLHANDLE callHandle,
                         pClient->async.callHandle = callHandle;
                         pClient->async.paParms    = paParms;
 
-                        Log(("svcCall: async.\n"));
+                        LogRel2(("svcCall: async.\n"));
                     }
 
                     vboxSvcClipboardUnlock ();
@@ -350,9 +441,9 @@ static DECLCALLBACK(void) svcCall (VBOXHGCMCALLHANDLE callHandle,
         case VBOX_SHARED_CLIPBOARD_FN_FORMATS:
         {
             /* The guest reports that some formats are available. */
-            Log(("svcCall: VBOX_SHARED_CLIPBOARD_FN_FORMATS\n"));
+            LogRel2(("svcCall: VBOX_SHARED_CLIPBOARD_FN_FORMATS\n"));
 
-            if (cParms != VBOX_HGCM_PARM_COUNT (VBoxClipboardFormats))
+            if (cParms != VBOX_SHARED_CLIPBOARD_CPARMS_FORMATS)
             {
                 rc = VERR_INVALID_PARAMETER;
             }
@@ -364,10 +455,10 @@ static DECLCALLBACK(void) svcCall (VBOXHGCMCALLHANDLE callHandle,
             else
             {
                 uint32_t u32Formats;
-                
+
                 rc = VBoxHGCMParmUInt32Get (&paParms[0], &u32Formats);
-                
-                if (VBOX_SUCCESS (rc))
+
+                if (RT_SUCCESS (rc))
                 {
                     if (   vboxSvcClipboardMode () != VBOX_SHARED_CLIPBOARD_MODE_GUEST_TO_HOST
                         && vboxSvcClipboardMode () != VBOX_SHARED_CLIPBOARD_MODE_BIDIRECTIONAL)
@@ -379,9 +470,9 @@ static DECLCALLBACK(void) svcCall (VBOXHGCMCALLHANDLE callHandle,
                     if (g_pfnExtension)
                     {
                         VBOXCLIPBOARDEXTPARMS parms;
-                        
+
                         parms.u32Format = u32Formats;
-                        
+
                         g_pfnExtension (g_pvExtension, VBOX_CLIPBOARD_EXT_FN_FORMAT_ANNOUNCE, &parms, sizeof (parms));
                     }
                     else
@@ -395,9 +486,9 @@ static DECLCALLBACK(void) svcCall (VBOXHGCMCALLHANDLE callHandle,
         case VBOX_SHARED_CLIPBOARD_FN_READ_DATA:
         {
             /* The guest wants to read data in the given format. */
-            Log(("svcCall: VBOX_SHARED_CLIPBOARD_FN_READ_DATA\n"));
+            LogRel2(("svcCall: VBOX_SHARED_CLIPBOARD_FN_READ_DATA\n"));
 
-            if (cParms != VBOX_HGCM_PARM_COUNT (VBoxClipboardReadData))
+            if (cParms != VBOX_SHARED_CLIPBOARD_CPARMS_READ_DATA)
             {
                 rc = VERR_INVALID_PARAMETER;
             }
@@ -415,12 +506,12 @@ static DECLCALLBACK(void) svcCall (VBOXHGCMCALLHANDLE callHandle,
                 uint32_t cb;
 
                 rc = VBoxHGCMParmUInt32Get (&paParms[0], &u32Format);
-                
-                if (VBOX_SUCCESS (rc))
+
+                if (RT_SUCCESS (rc))
                 {
                     rc = VBoxHGCMParmPtrGet (&paParms[1], &pv, &cb);
 
-                    if (VBOX_SUCCESS (rc))
+                    if (RT_SUCCESS (rc))
                     {
                         if (   vboxSvcClipboardMode () != VBOX_SHARED_CLIPBOARD_MODE_HOST_TO_GUEST
                             && vboxSvcClipboardMode () != VBOX_SHARED_CLIPBOARD_MODE_BIDIRECTIONAL)
@@ -428,30 +519,58 @@ static DECLCALLBACK(void) svcCall (VBOXHGCMCALLHANDLE callHandle,
                             rc = VERR_NOT_SUPPORTED;
                             break;
                         }
-                        
+
                         uint32_t cbActual = 0;
 
                         if (g_pfnExtension)
                         {
                             VBOXCLIPBOARDEXTPARMS parms;
-                            
+
                             parms.u32Format = u32Format;
-                            parms.pvData = pv;
+                            parms.u.pvData = pv;
                             parms.cbData = cb;
-                            
+
+                            g_fReadingData = true;
                             rc = g_pfnExtension (g_pvExtension, VBOX_CLIPBOARD_EXT_FN_DATA_READ, &parms, sizeof (parms));
-                            
-                            if (VBOX_SUCCESS (rc))
+                            LogRelFlow(("DATA: g_fDelayedAnnouncement = %d, g_u32DelayedFormats = 0x%x\n", g_fDelayedAnnouncement, g_u32DelayedFormats));
+                            if (g_fDelayedAnnouncement)
+                            {
+                                vboxSvcClipboardReportMsg (g_pClient, VBOX_SHARED_CLIPBOARD_HOST_MSG_FORMATS, g_u32DelayedFormats);
+                                g_fDelayedAnnouncement = false;
+                                g_u32DelayedFormats = 0;
+                            }
+                            g_fReadingData = false;
+
+                            if (RT_SUCCESS (rc))
                             {
                                 cbActual = parms.cbData;
                             }
                         }
                         else
                         {
+                            /* Release any other pending read, as we only
+                             * support one pending read at one time. */
+                            vboxSvcClipboardCompleteReadData(pClient, VERR_NO_DATA, 0);
                             rc = vboxClipboardReadData (pClient, u32Format, pv, cb, &cbActual);
                         }
-                        
-                        if (VBOX_SUCCESS (rc))
+
+                        /* Remember our read request until it is completed.
+                         * See the protocol description above for more
+                         * information. */
+                        if (rc == VINF_HGCM_ASYNC_EXECUTE)
+                        {
+                            if (vboxSvcClipboardLock())
+                            {
+                                pClient->asyncRead.callHandle = callHandle;
+                                pClient->asyncRead.paParms    = paParms;
+                                pClient->fReadPending         = true;
+                                fAsynchronousProcessing = true;
+                                vboxSvcClipboardUnlock();
+                            }
+                            else
+                                rc = VERR_NOT_SUPPORTED;
+                        }
+                        else if (RT_SUCCESS (rc))
                         {
                             VBoxHGCMParmUInt32Set (&paParms[2], cbActual);
                         }
@@ -463,9 +582,9 @@ static DECLCALLBACK(void) svcCall (VBOXHGCMCALLHANDLE callHandle,
         case VBOX_SHARED_CLIPBOARD_FN_WRITE_DATA:
         {
             /* The guest writes the requested data. */
-            Log(("svcCall: VBOX_SHARED_CLIPBOARD_FN_WRITE_DATA\n"));
+            LogRel2(("svcCall: VBOX_SHARED_CLIPBOARD_FN_WRITE_DATA\n"));
 
-            if (cParms != VBOX_HGCM_PARM_COUNT (VBoxClipboardWriteData))
+            if (cParms != VBOX_SHARED_CLIPBOARD_CPARMS_WRITE_DATA)
             {
                 rc = VERR_INVALID_PARAMETER;
             }
@@ -483,11 +602,11 @@ static DECLCALLBACK(void) svcCall (VBOXHGCMCALLHANDLE callHandle,
 
                 rc = VBoxHGCMParmUInt32Get (&paParms[0], &u32Format);
 
-                if (VBOX_SUCCESS (rc))
+                if (RT_SUCCESS (rc))
                 {
                     rc = VBoxHGCMParmPtrGet (&paParms[1], &pv, &cb);
 
-                    if (VBOX_SUCCESS (rc))
+                    if (RT_SUCCESS (rc))
                     {
                         if (   vboxSvcClipboardMode () != VBOX_SHARED_CLIPBOARD_MODE_GUEST_TO_HOST
                             && vboxSvcClipboardMode () != VBOX_SHARED_CLIPBOARD_MODE_BIDIRECTIONAL)
@@ -499,11 +618,11 @@ static DECLCALLBACK(void) svcCall (VBOXHGCMCALLHANDLE callHandle,
                         if (g_pfnExtension)
                         {
                             VBOXCLIPBOARDEXTPARMS parms;
-                            
+
                             parms.u32Format = u32Format;
-                            parms.pvData = pv;
+                            parms.u.pvData = pv;
                             parms.cbData = cb;
-                            
+
                             g_pfnExtension (g_pvExtension, VBOX_CLIPBOARD_EXT_FN_DATA_WRITE, &parms, sizeof (parms));
                         }
                         else
@@ -521,7 +640,7 @@ static DECLCALLBACK(void) svcCall (VBOXHGCMCALLHANDLE callHandle,
         }
     }
 
-    LogFlow(("svcCall: rc = %Vrc\n", rc));
+    LogRelFlow(("svcCall: rc = %Rrc\n", rc));
 
     if (!fAsynchronousProcessing)
     {
@@ -529,23 +648,47 @@ static DECLCALLBACK(void) svcCall (VBOXHGCMCALLHANDLE callHandle,
     }
 }
 
+/** If the client in the guest is waiting for a read operation to complete
+ * then complete it, otherwise return.  See the protocol description in the
+ * shared clipboard module description. */
+void vboxSvcClipboardCompleteReadData(VBOXCLIPBOARDCLIENTDATA *pClient, int rc, uint32_t cbActual)
+{
+    VBOXHGCMCALLHANDLE callHandle = NULL;
+    VBOXHGCMSVCPARM *paParms = NULL;
+    bool fReadPending = false;
+    if (vboxSvcClipboardLock())  /* if not can we do anything useful? */
+    {
+        callHandle   = pClient->asyncRead.callHandle;
+        paParms      = pClient->asyncRead.paParms;
+        fReadPending = pClient->fReadPending;
+        pClient->fReadPending = false;
+        vboxSvcClipboardUnlock();
+    }
+    if (fReadPending)
+    {
+        VBoxHGCMParmUInt32Set (&paParms[2], cbActual);
+        g_pHelpers->pfnCallComplete (callHandle, rc);
+    }
+}
+
 /*
  * We differentiate between a function handler for the guest and one for the host.
  */
-static DECLCALLBACK(int) svcHostCall (uint32_t u32Function,
+static DECLCALLBACK(int) svcHostCall (void *,
+                                      uint32_t u32Function,
                                       uint32_t cParms,
                                       VBOXHGCMSVCPARM paParms[])
 {
     int rc = VINF_SUCCESS;
 
-    Log(("svcHostCall: fn = %d, cParms = %d, pparms = %d\n",
+    LogRel2(("svcHostCall: fn = %d, cParms = %d, pparms = %d\n",
          u32Function, cParms, paParms));
 
     switch (u32Function)
     {
         case VBOX_SHARED_CLIPBOARD_HOST_FN_SET_MODE:
         {
-            Log(("svcCall: VBOX_SHARED_CLIPBOARD_HOST_FN_SET_MODE\n"));
+            LogRel2(("svcCall: VBOX_SHARED_CLIPBOARD_HOST_FN_SET_MODE\n"));
 
             if (cParms != 1)
             {
@@ -571,16 +714,54 @@ static DECLCALLBACK(int) svcHostCall (uint32_t u32Function,
             break;
     }
 
-    LogFlow(("svcHostCall: rc = %Vrc\n", rc));
+    LogRelFlow(("svcHostCall: rc = %Rrc\n", rc));
     return rc;
 }
 
-static DECLCALLBACK(int) svcSaveState(uint32_t u32ClientID, void *pvClient, PSSMHANDLE pSSM)
+/** This structure corresponds to the original layout of the
+ * VBOXCLIPBOARDCLIENTDATA structure.  As the structure was saved as a whole
+ * when saving state, we need to remember it forever in order to preserve
+ * compatibility.
+ * @todo the first person who needs to make an incompatible change to the
+ *       saved state should switch to saving individual data members.  So far,
+ *       there are only three we care about anyway! */
+typedef struct _CLIPSAVEDSTATEDATA
+{
+    struct _CLIPSAVEDSTATEDATA *pNext;
+    struct _CLIPSAVEDSTATEDATA *pPrev;
+
+    VBOXCLIPBOARDCONTEXT *pCtx;
+
+    uint32_t u32ClientID;
+
+    bool fAsync: 1; /* Guest is waiting for a message. */
+
+    bool fMsgQuit: 1;
+    bool fMsgReadData: 1;
+    bool fMsgFormats: 1;
+
+    struct {
+        VBOXHGCMCALLHANDLE callHandle;
+        VBOXHGCMSVCPARM *paParms;
+    } async;
+
+    struct {
+         void *pv;
+         uint32_t cb;
+         uint32_t u32Format;
+    } data;
+
+    uint32_t u32AvailableFormats;
+    uint32_t u32RequestedFormat;
+
+} CLIPSAVEDSTATEDATA;
+
+static DECLCALLBACK(int) svcSaveState(void *, uint32_t u32ClientID, void *pvClient, PSSMHANDLE pSSM)
 {
     /* If there are any pending requests, they must be completed here. Since
      * the service is single threaded, there could be only requests
      * which the service itself has postponed.
-     * 
+     *
      * HGCM knows that the state is being saved and that the pfnComplete
      * calls are just clean ups. These requests are saved by the VMMDev.
      *
@@ -588,15 +769,21 @@ static DECLCALLBACK(int) svcSaveState(uint32_t u32ClientID, void *pvClient, PSSM
      * by VMMDev. The service therefore must save state as if there were no
      * pending request.
      */
-    Log(("svcSaveState: u32ClientID = %d\n", u32ClientID));
+    LogRel2(("svcSaveState: u32ClientID = %d\n", u32ClientID));
 
     VBOXCLIPBOARDCLIENTDATA *pClient = (VBOXCLIPBOARDCLIENTDATA *)pvClient;
 
+    CLIPSAVEDSTATEDATA savedState = { 0 };
     /* Save client structure length & contents */
-    int rc = SSMR3PutU32(pSSM, sizeof(*pClient));
+    int rc = SSMR3PutU32(pSSM, sizeof(savedState));
     AssertRCReturn(rc, rc);
 
-    rc = SSMR3PutMem(pSSM, pClient, sizeof(*pClient));
+    savedState.u32ClientID        = pClient->u32ClientID;
+    savedState.fMsgQuit           = pClient->fMsgQuit;
+    savedState.fMsgReadData       = pClient->fMsgReadData;
+    savedState.fMsgFormats        = pClient->fMsgFormats;
+    savedState.u32RequestedFormat = pClient->u32RequestedFormat;
+    rc = SSMR3PutMem(pSSM, &savedState, sizeof(savedState));
     AssertRCReturn(rc, rc);
 
     if (pClient->fAsync)
@@ -605,12 +792,14 @@ static DECLCALLBACK(int) svcSaveState(uint32_t u32ClientID, void *pvClient, PSSM
         pClient->fAsync = false;
     }
 
+    vboxSvcClipboardCompleteReadData(pClient, VINF_SUCCESS, 0);
+
     return VINF_SUCCESS;
 }
 
-static DECLCALLBACK(int) svcLoadState(uint32_t u32ClientID, void *pvClient, PSSMHANDLE pSSM)
+static DECLCALLBACK(int) svcLoadState(void *, uint32_t u32ClientID, void *pvClient, PSSMHANDLE pSSM)
 {
-    Log(("svcLoadState: u32ClientID = %d\n", u32ClientID));
+    LogRel2(("svcLoadState: u32ClientID = %d\n", u32ClientID));
 
     VBOXCLIPBOARDCLIENTDATA *pClient = (VBOXCLIPBOARDCLIENTDATA *)pvClient;
 
@@ -622,27 +811,27 @@ static DECLCALLBACK(int) svcLoadState(uint32_t u32ClientID, void *pvClient, PSSM
     int rc = SSMR3GetU32(pSSM, &len);
     AssertRCReturn(rc, rc);
 
-    if (len != sizeof(VBOXCLIPBOARDCLIENTDATA))
+    if (len != sizeof(CLIPSAVEDSTATEDATA))
     {
-        Log(("Client len mismatch: %d %d\n", len, sizeof (VBOXCLIPBOARDCLIENTDATA)));
+        LogRel2(("Client data size mismatch: expected %d, got %d\n", sizeof (CLIPSAVEDSTATEDATA), len));
         return VERR_SSM_DATA_UNIT_FORMAT_CHANGED;
     }
 
-    VBOXCLIPBOARDCLIENTDATA client;
-    rc = SSMR3GetMem(pSSM, &client, sizeof(client));
+    CLIPSAVEDSTATEDATA savedState;
+    rc = SSMR3GetMem(pSSM, &savedState, sizeof(savedState));
     AssertRCReturn(rc, rc);
 
     /* Verify the loaded clients data and update the pClient. */
-    if (pClient->u32ClientID != client.u32ClientID)
+    if (pClient->u32ClientID != savedState.u32ClientID)
     {
-        Log(("Client ID mismatch: %d %d\n", pClient->u32ClientID, client.u32ClientID));
+        LogRel2(("Client ID mismatch: expected %d, got %d\n", pClient->u32ClientID, savedState.u32ClientID));
         return VERR_SSM_DATA_UNIT_FORMAT_CHANGED;
     }
 
-    pClient->fMsgQuit     = client.fMsgQuit;
-    pClient->fMsgReadData = client.fMsgReadData;
-    pClient->fMsgFormats  = client.fMsgFormats;
-    pClient->u32RequestedFormat = client.u32RequestedFormat;
+    pClient->fMsgQuit           = savedState.fMsgQuit;
+    pClient->fMsgReadData       = savedState.fMsgReadData;
+    pClient->fMsgFormats        = savedState.fMsgFormats;
+    pClient->u32RequestedFormat = savedState.u32RequestedFormat;
 
     /* Actual host data are to be reported to guest (SYNC). */
     vboxClipboardSync (pClient);
@@ -658,50 +847,59 @@ static DECLCALLBACK(int) extCallback (uint32_t u32Function, uint32_t u32Format,
         {
             case VBOX_CLIPBOARD_EXT_FN_FORMAT_ANNOUNCE:
             {
-                vboxSvcClipboardReportMsg (g_pClient, VBOX_SHARED_CLIPBOARD_HOST_MSG_FORMATS, u32Format);
+                LogRelFlow(("ANNOUNCE: g_fReadingData = %d\n", g_fReadingData));
+                if (g_fReadingData)
+                {
+                    g_fDelayedAnnouncement = true;
+                    g_u32DelayedFormats = u32Format;
+                }
+                else
+                {
+                    vboxSvcClipboardReportMsg (g_pClient, VBOX_SHARED_CLIPBOARD_HOST_MSG_FORMATS, u32Format);
+                }
             } break;
 
             case VBOX_CLIPBOARD_EXT_FN_DATA_READ:
             {
                 vboxSvcClipboardReportMsg (g_pClient, VBOX_SHARED_CLIPBOARD_HOST_MSG_READ_DATA, u32Format);
             } break;
-        
+
             default:
                 return VERR_NOT_SUPPORTED;
         }
     }
-    
+
     return VINF_SUCCESS;
 }
 
-static DECLCALLBACK(int) svcRegisterExtension(PFNHGCMSVCEXT pfnExtension, void *pvExtension)
+static DECLCALLBACK(int) svcRegisterExtension(void *, PFNHGCMSVCEXT pfnExtension, void *pvExtension)
 {
-    LogFlowFunc(("pfnExtension = %p\n", pfnExtension));
-    
+    LogRelFlowFunc(("pfnExtension = %p\n", pfnExtension));
+
     VBOXCLIPBOARDEXTPARMS parms;
-    
+
     if (pfnExtension)
     {
         /* Install extension. */
         g_pfnExtension = pfnExtension;
         g_pvExtension = pvExtension;
-        
-        parms.pvData = (void *)extCallback;
+
+        parms.u.pfnCallback = extCallback;
         g_pfnExtension (g_pvExtension, VBOX_CLIPBOARD_EXT_FN_SET_CALLBACK, &parms, sizeof (parms));
     }
     else
     {
         if (g_pfnExtension)
         {
-            parms.pvData = NULL;
+            parms.u.pfnCallback = NULL;
             g_pfnExtension (g_pvExtension, VBOX_CLIPBOARD_EXT_FN_SET_CALLBACK, &parms, sizeof (parms));
         }
-        
+
         /* Uninstall extension. */
         g_pfnExtension = NULL;
         g_pvExtension = NULL;
     }
-    
+
     return VINF_SUCCESS;
 }
 
@@ -709,7 +907,7 @@ extern "C" DECLCALLBACK(DECLEXPORT(int)) VBoxHGCMSvcLoad (VBOXHGCMSVCFNTABLE *pt
 {
     int rc = VINF_SUCCESS;
 
-    LogFlowFunc(("ptable = %p\n", ptable));
+    LogRelFlowFunc(("ptable = %p\n", ptable));
 
     if (!ptable)
     {
@@ -717,7 +915,7 @@ extern "C" DECLCALLBACK(DECLEXPORT(int)) VBoxHGCMSvcLoad (VBOXHGCMSVCFNTABLE *pt
     }
     else
     {
-        Log(("VBoxHGCMSvcLoad: ptable->cbSize = %d, ptable->u32Version = 0x%08X\n", ptable->cbSize, ptable->u32Version));
+        LogRel2(("VBoxHGCMSvcLoad: ptable->cbSize = %d, ptable->u32Version = 0x%08X\n", ptable->cbSize, ptable->u32Version));
 
         if (   ptable->cbSize != sizeof (VBOXHGCMSVCFNTABLE)
             || ptable->u32Version != VBOX_HGCM_SVC_VERSION)
@@ -738,6 +936,7 @@ extern "C" DECLCALLBACK(DECLEXPORT(int)) VBoxHGCMSvcLoad (VBOXHGCMSVCFNTABLE *pt
             ptable->pfnSaveState  = svcSaveState;
             ptable->pfnLoadState  = svcLoadState;
             ptable->pfnRegisterExtension  = svcRegisterExtension;
+            ptable->pvService     = NULL;
 
             /* Service specific initialization. */
             rc = svcInit ();
