@@ -180,7 +180,8 @@ PerformanceCollector::COMGETTER(MetricNames) (ComSafeArrayOut (BSTR, theMetricNa
     com::SafeArray <BSTR> metricNames(RT_ELEMENTS(gMetricNames));
     for (size_t i = 0; i < RT_ELEMENTS(gMetricNames); i++)
     {
-        gMetricNames[i].detachTo(&metricNames[i]);
+        Bstr tmp(gMetricNames[i]); /* gcc-3.3 cruft */
+        tmp.cloneTo(&metricNames[i]);
     }
     //gMetricNames.detachTo(ComSafeArrayOutArg (theMetricNames));
     metricNames.detachTo (ComSafeArrayOutArg (theMetricNames));
@@ -521,6 +522,25 @@ void PerformanceCollector::unregisterMetricsFor (const ComPtr <IUnknown> &aObjec
     //LogFlowThisFuncLeave();
 }
 
+void PerformanceCollector::suspendSampling()
+{
+    AutoCaller autoCaller (this);
+    if (!SUCCEEDED (autoCaller.rc())) return;
+
+    int rc = RTTimerLRStop(m.sampler);
+    AssertRC(rc);
+}
+
+void PerformanceCollector::resumeSampling()
+{
+    AutoCaller autoCaller (this);
+    if (!SUCCEEDED (autoCaller.rc())) return;
+
+    int rc = RTTimerLRStart(m.sampler, 0);
+    AssertRC(rc);
+}
+
+
 // private methods
 ///////////////////////////////////////////////////////////////////////////////
 
