@@ -28,15 +28,16 @@
 #include <iprt/env.h>
 #include <VBox/log.h>
 
-#include "VBoxCAPI_v2_2.h"
+#include "VBoxCAPI_v3_0.h"
 #include "VBox/com/com.h"
 #include "VBox/version.h"
 
 using namespace std;
 
-static ISession            *Session     = NULL;
-static IVirtualBox         *Ivirtualbox = NULL;
-static nsIComponentManager *manager     = NULL;
+static ISession            *Session        = NULL;
+static IVirtualBox         *Ivirtualbox    = NULL;
+static nsIComponentManager *manager        = NULL;
+static nsIEventQueue       *eventQ         = NULL;
 
 static void VBoxComUninitialize(void);
 
@@ -74,17 +75,24 @@ VBoxComUnallocMem(void *ptr)
 }
 
 static void
-VBoxComInitialize(IVirtualBox **virtualBox, ISession **session)
+VBoxComInitialize(const char *pszVirtualBoxIID, IVirtualBox **ppVirtualBox,
+                  const char *pszSessionIID, ISession **ppSession)
 {
     nsresult rc;
+    nsID virtualBoxIID;
+    nsID sessionIID;
 
-    *session    = NULL;
-    *virtualBox = NULL;
+    *ppSession    = NULL;
+    *ppVirtualBox = NULL;
+
+    /* convert the string representation of UUID to nsIID type */
+    if (!(virtualBoxIID.Parse(pszVirtualBoxIID) && sessionIID.Parse(pszSessionIID)))
+        return;
 
     rc = com::Initialize();
     if (NS_FAILED(rc))
     {
-        Log(("Cbinding: XPCOM could not be initialized! rc=%Rhrc\n",rc));
+        Log(("Cbinding: XPCOM could not be initialized! rc=%Rhrc\n", rc));
         VBoxComUninitialize();
         return;
     }
@@ -92,15 +100,23 @@ VBoxComInitialize(IVirtualBox **virtualBox, ISession **session)
     rc = NS_GetComponentManager (&manager);
     if (NS_FAILED(rc))
     {
-        Log(("Cbinding: Could not get component manager! rc=%Rhrc\n",rc));
+        Log(("Cbinding: Could not get component manager! rc=%Rhrc\n", rc));
+        VBoxComUninitialize();
+        return;
+    }
+
+    rc = NS_GetMainEventQ (&eventQ);
+    if (NS_FAILED(rc))
+    {
+        Log(("Cbinding: Could not get xpcom event queue! rc=%Rhrc\n", rc));
         VBoxComUninitialize();
         return;
     }
 
     rc = manager->CreateInstanceByContractID(NS_VIRTUALBOX_CONTRACTID,
                                              nsnull,
-                                             NS_GET_IID(IVirtualBox),
-                                             (void **)virtualBox);
+                                             virtualBoxIID,
+                                             (void **)ppVirtualBox);
     if (NS_FAILED(rc))
     {
         Log(("Cbinding: Could not instantiate VirtualBox object! rc=%Rhrc\n",rc));
@@ -112,8 +128,8 @@ VBoxComInitialize(IVirtualBox **virtualBox, ISession **session)
 
     rc = manager->CreateInstanceByContractID (NS_SESSION_CONTRACTID,
                                               nsnull,
-                                              NS_GET_IID(ISession),
-                                              (void **)session);
+                                              sessionIID,
+                                              (void **)ppSession);
     if (NS_FAILED(rc))
     {
         Log(("Cbinding: Could not instantiate Session object! rc=%Rhrc\n",rc));
@@ -123,14 +139,22 @@ VBoxComInitialize(IVirtualBox **virtualBox, ISession **session)
 
     Log(("Cbinding: ISession object created.\n"));
 
-    /* Store session & virtualBox so that VBoxComUninitialize
+    /* Store ppSession & ppVirtualBox so that VBoxComUninitialize
      * can later take care of them while cleanup
      */
-    Session     = *session;
-    Ivirtualbox = *virtualBox;
+    Session     = *ppSession;
+    Ivirtualbox = *ppVirtualBox;
 
 }
 
+static void
+VBoxComInitializeV1(IVirtualBox **ppVirtualBox, ISession **ppSession)
+{
+    /* stub that always fails. */
+    *ppVirtualBox = NULL;
+    *ppSession = NULL;
+}
+
 static void
 VBoxComUninitialize(void)
 {
@@ -138,12 +162,20 @@ VBoxComUninitialize(void)
         NS_RELEASE(Session);        // decrement refcount
     if (Ivirtualbox)
         NS_RELEASE(Ivirtualbox);    // decrement refcount
+    if (eventQ)
+        NS_RELEASE(eventQ);         // decrement refcount
     if (manager)
         NS_RELEASE(manager);        // decrement refcount
     com::Shutdown();
     Log(("Cbinding: Cleaned up the created IVirtualBox and ISession Objects.\n"));
 }
 
+static void
+VBoxGetEventQueue(nsIEventQueue **eventQueue)
+{
+    *eventQueue = eventQ;
+}
+
 static uint32_t
 VBoxVersion(void)
 {
@@ -157,7 +189,9 @@ VBoxVersion(void)
 VBOXXPCOMC_DECL(PCVBOXXPCOM)
 VBoxGetXPCOMCFunctions(unsigned uVersion)
 {
-    /* The current version. */
+    /*
+     * The current interface version.
+     */
     static const VBOXXPCOMC s_Functions =
     {
         sizeof(VBOXXPCOMC),
@@ -175,13 +209,65 @@ VBoxGetXPCOMCFunctions(unsigned uVersion)
         VBoxUtf16ToUtf8,
         VBoxUtf8ToUtf16,
 
+        VBoxGetEventQueue,
+
         VBOX_XPCOMC_VERSION
     };
 
-    if ((uVersion & 0xffff0000U) != VBOX_XPCOMC_VERSION)
-        return NULL; /* not supported. */
+    if ((uVersion & 0xffff0000U) == (VBOX_XPCOMC_VERSION & 0xffff0000U))
+        return &s_Functions;
 
-    return &s_Functions;
+    /*
+     * Legacy interface version 1.0.
+     */
+    static const struct VBOXXPCOMCV1
+    {
+        /** The size of the structure. */
+        unsigned cb;
+        /** The structure version. */
+        unsigned uVersion;
+
+        unsigned int (*pfnGetVersion)(void);
+
+        void  (*pfnComInitialize)(IVirtualBox **virtualBox, ISession **session);
+        void  (*pfnComUninitialize)(void);
+
+        void  (*pfnComUnallocMem)(void *pv);
+        void  (*pfnUtf16Free)(PRUnichar *pwszString);
+        void  (*pfnUtf8Free)(char *pszString);
+
+        int   (*pfnUtf16ToUtf8)(const PRUnichar *pwszString, char **ppszString);
+        int   (*pfnUtf8ToUtf16)(const char *pszString, PRUnichar **ppwszString);
+
+        /** Tail version, same as uVersion. */
+        unsigned uEndVersion;
+    } s_Functions_v1_0 =
+    {
+        sizeof(s_Functions_v1_0),
+        0x00010000U,
+
+        VBoxVersion,
+
+        VBoxComInitializeV1,
+        VBoxComUninitialize,
+
+        VBoxComUnallocMem,
+        VBoxUtf16Free,
+        VBoxUtf8Free,
+
+        VBoxUtf16ToUtf8,
+        VBoxUtf8ToUtf16,
+
+        0x00010000U
+    };
+
+    if ((uVersion & 0xffff0000U) == 0x00010000U)
+        return (PCVBOXXPCOM)&s_Functions_v1_0;
+
+    /*
+     * Unsupported interface version.
+     */
+    return NULL;
 }
 
 /* vim: set ts=4 sw=4 et: */
