@@ -32,7 +32,6 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #include "the-solaris-kernel.h"
-#include <time.h>
 
 #include <iprt/semaphore.h>
 #include <iprt/alloc.h>
@@ -78,7 +77,7 @@ RTDECL(int)  RTSemEventCreate(PRTSEMEVENT pEventSem)
         pEventInt->cWaiters = 0;
         pEventInt->cWaking = 0;
         pEventInt->fSignaled = 0;
-        mutex_init(&pEventInt->Mtx, "IPRT Event Semaphore", MUTEX_DRIVER, NULL);
+        mutex_init(&pEventInt->Mtx, "IPRT Event Semaphore", MUTEX_DRIVER, (void *)ipltospl(DISP_LEVEL));
         cv_init(&pEventInt->Cnd, "IPRT CV", CV_DRIVER, NULL);
         *pEventSem = pEventInt;
         return VINF_SUCCESS;
