@@ -33,6 +33,7 @@
 #include <iprt/memobj.h>
 #include <iprt/cpuset.h>
 #include <iprt/mp.h>
+#include <iprt/avl.h>
 
 #if HC_ARCH_BITS == 64 || defined(VBOX_WITH_HYBRID_32BIT_KERNEL) || defined (VBOX_WITH_64_BITS_GUESTS)
 /* Enable 64 bits guest support. */
@@ -44,7 +45,18 @@
 #define HWACCM_VTX_WITH_EPT
 #define HWACCM_VTX_WITH_VPID
 
-__BEGIN_DECLS
+
+#if 0
+/* Seeing somewhat random behaviour on my Nehalem system with auto-save of guest MSRs;
+ * for some strange reason the CPU doesn't save the MSRs during the VM-exit.
+ * Clearly visible with a dual VCPU configured OpenSolaris 200906 live cd VM.
+ *
+ * Note: change the assembly files when enabling this! (remove the manual auto load/save)
+ */
+#define VBOX_WITH_AUTO_MSR_LOAD_RESTORE
+#endif
+
+RT_C_DECLS_BEGIN
 
 
 /** @defgroup grp_hwaccm_int       Internal
@@ -120,8 +132,8 @@ __BEGIN_DECLS
 /** @} */
 
 
-/** Maxium resume loops allowed in ring 0 (safety precaution) */
-#define HWACCM_MAX_RESUME_LOOPS             1024
+/** Maximum number of page flushes we are willing to remember before considering a full TLB flush. */
+#define HWACCM_MAX_TLB_SHOOTDOWN_PAGES      8
 
 /** Size for the EPT identity page table (1024 4 MB pages to cover the entire address space). */
 #define HWACCM_EPT_IDENTITY_PG_TABLE_SIZE   PAGE_SIZE
@@ -130,9 +142,18 @@ __BEGIN_DECLS
 /** Total guest mapped memory needed. */
 #define HWACCM_VTX_TOTAL_DEVHEAP_MEM        (HWACCM_EPT_IDENTITY_PG_TABLE_SIZE + HWACCM_VTX_TSS_SIZE)
 
+/* Enable for TPR guest patching. */
+#define VBOX_HWACCM_WITH_GUEST_PATCHING
+
 /** HWACCM SSM version
  */
+#ifdef VBOX_HWACCM_WITH_GUEST_PATCHING
+#define HWACCM_SSM_VERSION                  5
+#define HWACCM_SSM_VERSION_NO_PATCHING      4
+#else
 #define HWACCM_SSM_VERSION                  4
+#define HWACCM_SSM_VERSION_NO_PATCHING      4
+#endif
 #define HWACCM_SSM_VERSION_2_0_X            3
 
 /* Per-cpu information. (host) */
@@ -168,6 +189,56 @@ typedef union
     uint64_t            u;
 } VMX_CAPABILITY;
 
+typedef enum
+{
+    HWACCMPENDINGIO_INVALID = 0,
+    HWACCMPENDINGIO_PORT_READ,
+    HWACCMPENDINGIO_PORT_WRITE,
+    HWACCMPENDINGIO_STRING_READ,
+    HWACCMPENDINGIO_STRING_WRITE,
+    /** The usual 32-bit paranoia. */
+    HWACCMPENDINGIO_32BIT_HACK   = 0x7fffffff
+} HWACCMPENDINGIO;
+
+
+typedef enum
+{
+    HWACCMTPRINSTR_INVALID,
+    HWACCMTPRINSTR_READ,
+    HWACCMTPRINSTR_READ_SHR4,
+    HWACCMTPRINSTR_WRITE_REG,
+    HWACCMTPRINSTR_WRITE_IMM,
+    HWACCMTPRINSTR_JUMP_REPLACEMENT,
+    /** The usual 32-bit paranoia. */
+    HWACCMTPRINSTR_32BIT_HACK   = 0x7fffffff
+} HWACCMTPRINSTR;
+
+typedef struct
+{
+    /** The key is the address of patched instruction. (32 bits GC ptr) */
+    AVLOU32NODECORE         Core;
+    /** Original opcode. */
+    uint8_t                 aOpcode[16];
+    /** Instruction size. */
+    uint32_t                cbOp;
+    /** Replacement opcode. */
+    uint8_t                 aNewOpcode[16];
+    /** Replacement instruction size. */
+    uint32_t                cbNewOp;
+    /** Instruction type. */
+    HWACCMTPRINSTR          enmType;
+    /** Source operand. */
+    uint32_t                uSrcOperand;
+    /** Destination operand. */
+    uint32_t                uDstOperand;
+    /** Number of times the instruction caused a fault. */
+    uint32_t                cFaults;
+    /** Patch address of the jump replacement. */
+    RTGCPTR32               pJumpTarget;
+} HWACCMTPRPATCH;
+/** Pointer to HWACCMTPRPATCH. */
+typedef HWACCMTPRPATCH *PHWACCMTPRPATCH;
+
 /**
  * Switcher function, HC to RC.
  *
@@ -187,9 +258,6 @@ typedef struct HWACCM
     /** Set when we've initialized VMX or SVM. */
     bool                        fInitialized;
 
-    /** Set when we're using VMX/SVN at that moment. */
-    bool                        fActive;
-
     /** Set when hardware acceleration is allowed. */
     bool                        fAllowed;
 
@@ -199,12 +267,15 @@ typedef struct HWACCM
     /** Set if nested paging is allowed. */
     bool                        fAllowNestedPaging;
 
-    /** Set if we're supposed to inject an NMI. */
-    bool                        fInjectNMI;
-
     /** Set if we can support 64-bit guests or not. */
     bool                        fAllow64BitGuests;
 
+    /** Set if an IO-APIC is configured for this VM. */
+    bool                        fHasIoApic;
+
+    /** Set when TPR patching is allowed. */
+    bool                        fTRPPatchingAllowed;
+
     /** Explicit alignment padding to make 32-bit gcc align u64RegisterMask
      *  naturally. */
     bool                        padding[1];
@@ -215,6 +286,18 @@ typedef struct HWACCM
     /** Maximum ASID allowed. */
     RTUINT                      uMaxASID;
 
+    /** The maximum number of resumes loops allowed in ring-0 (safety precaution).
+     * This number is set much higher when RTThreadPreemptIsPending is reliable. */
+    uint32_t                    cMaxResumeLoops;
+
+    /** Guest allocated memory for patching purposes. */
+    RTGCPTR                     pGuestPatchMem;
+    /** Current free pointer inside the patch block. */
+    RTGCPTR                     pFreeGuestPatchMem;
+    /** Size of the guest patch memory block. */
+    uint32_t                    cbGuestPatchMem;
+    uint32_t                    uPadding1;
+
 #if HC_ARCH_BITS == 32 && defined(VBOX_ENABLE_64_BITS_GUESTS) && !defined(VBOX_WITH_HYBRID_32BIT_KERNEL)
     /** 32 to 64 bits switcher entrypoint. */
     R0PTRTYPE(PFNHWACCMSWITCHERHC) pfnHost32ToGuest64R0;
@@ -234,9 +317,9 @@ typedef struct HWACCM
     /* Test handler */
     RTRCPTR                     pfnTest64;
 
-    RTRCPTR                     uAlignment[1];
-#elif defined(VBOX_WITH_HYBRID_32BIT_KERNEL)
-    uint32_t                    u32Alignment[1];
+    RTRCPTR                     uAlignment[2];
+/*#elif defined(VBOX_WITH_HYBRID_32BIT_KERNEL)
+    uint32_t                    u32Alignment[1]; */
 #endif
 
     struct
@@ -259,20 +342,13 @@ typedef struct HWACCM
         /** Virtual address of the identity page table used for real mode and protected mode without paging emulation in EPT mode. */
         R3PTRTYPE(PX86PD)           pNonPagingModeEPTPageTable;
 
-        /** R0 memory object for the virtual APIC mmio cache. */
+        /** R0 memory object for the APIC physical page (serves for filtering accesses). */
         RTR0MEMOBJ                  pMemObjAPIC;
-        /** Physical address of the virtual APIC mmio cache. */
+        /** Physical address of the APIC physical page (serves for filtering accesses). */
         RTHCPHYS                    pAPICPhys;
-        /** Virtual address of the virtual APIC mmio cache. */
+        /** Virtual address of the APIC physical page (serves for filtering accesses). */
         R0PTRTYPE(uint8_t *)        pAPIC;
 
-        /** R0 memory object for the MSR bitmap (1 page). */
-        RTR0MEMOBJ                  pMemObjMSRBitmap;
-        /** Physical address of the MSR bitmap (1 page). */
-        RTHCPHYS                    pMSRBitmapPhys;
-        /** Virtual address of the MSR bitmap (1 page). */
-        R0PTRTYPE(uint8_t *)        pMSRBitmap;
-
         /** R0 memory object for the MSR entry load page (guest MSRs). */
         RTR0MEMOBJ                  pMemObjMSREntryLoad;
         /** Physical address of the MSR entry load page (guest MSRs). */
@@ -337,16 +413,8 @@ typedef struct HWACCM
         bool                        fEnabled;
         /** Set if erratum 170 affects the AMD cpu. */
         bool                        fAlwaysFlushTLB;
-        /** Explicit alignment padding to make 32-bit gcc align u64RegisterMask
-         *  naturally. */
-        bool                        padding[1];
-
-        /** R0 memory object for the host VM control block (VMCB). */
-        RTR0MEMOBJ                  pMemObjVMCBHost;
-        /** Physical address of the host VM control block (VMCB). */
-        RTHCPHYS                    pVMCBHostPhys;
-        /** Virtual address of the host VM control block (VMCB). */
-        R0PTRTYPE(void *)           pVMCBHost;
+        /** Set when TPR patching is active. */
+        bool                        fTPRPatchingActive;
 
         /** R0 memory object for the IO bitmap (12kb). */
         RTR0MEMOBJ                  pMemObjIOBitmap;
@@ -355,18 +423,18 @@ typedef struct HWACCM
         /** Virtual address of the IO bitmap. */
         R0PTRTYPE(void *)           pIOBitmap;
 
-        /** R0 memory object for the MSR bitmap (8kb). */
-        RTR0MEMOBJ                  pMemObjMSRBitmap;
-        /** Physical address of the MSR bitmap (8kb). */
-        RTHCPHYS                    pMSRBitmapPhys;
-        /** Virtual address of the MSR bitmap. */
-        R0PTRTYPE(void *)           pMSRBitmap;
-
         /** SVM revision. */
         uint32_t                    u32Rev;
 
         /** SVM feature bits from cpuid 0x8000000a */
         uint32_t                    u32Features;
+
+        /**
+         * AVL tree with all patches (active or disabled) sorted by guest instruction address
+         */
+        AVLOU32TREE                 PatchTree;
+        uint32_t                    cPatches;
+        HWACCMTPRPATCH              aPatches[64];
     } svm;
 
     struct
@@ -380,6 +448,12 @@ typedef struct HWACCM
 
     /** HWACCMR0Init was run */
     bool                    fHWACCMR0Init;
+    bool                    u8Alignment[7];
+
+    STAMCOUNTER             StatTPRPatchSuccess;
+    STAMCOUNTER             StatTPRPatchFailure;
+    STAMCOUNTER             StatTPRReplaceSuccess;
+    STAMCOUNTER             StatTPRReplaceFailure;
 } HWACCM;
 /** Pointer to HWACCM VM instance data. */
 typedef HWACCM *PHWACCM;
@@ -446,6 +520,16 @@ typedef struct VMCSCACHE
 /** Pointer to VMCSCACHE. */
 typedef VMCSCACHE *PVMCSCACHE;
 
+/** VMX StartVM function. */
+typedef DECLCALLBACK(int) FNHWACCMVMXSTARTVM(RTHCUINT fResume, PCPUMCTX pCtx, PVMCSCACHE pCache, PVM pVM, PVMCPU pVCpu);
+/** Pointer to a VMX StartVM function. */
+typedef R0PTRTYPE(FNHWACCMVMXSTARTVM *) PFNHWACCMVMXSTARTVM;
+
+/** SVM VMRun function. */
+typedef DECLCALLBACK(int) FNHWACCMSVMVMRUN(RTHCPHYS pVMCBHostPhys, RTHCPHYS pVMCBPhys, PCPUMCTX pCtx, PVM pVM, PVMCPU pVCpu);
+/** Pointer to a SVM VMRun function. */
+typedef R0PTRTYPE(FNHWACCMSVMVMRUN *) PFNHWACCMSVMVMRUN;
+
 /**
  * HWACCM VMCPU Instance data.
  */
@@ -460,9 +544,8 @@ typedef struct HWACCMCPU
     /** Set if we need to flush the TLB during the world switch. */
     bool                        fForceTLBFlush;
 
-    /** Explicit alignment padding to make 32-bit gcc align u64RegisterMask
-     *  naturally. */
-    bool                        padding[1];
+    /** Set when we're using VT-x or AMD-V at that moment. */
+    bool                        fActive;
 
     /** HWACCM_CHANGED_* flags. */
     RTUINT                      fContextUseFlags;
@@ -478,19 +561,29 @@ typedef struct HWACCMCPU
 
     struct
     {
-        /** R0 memory object for the VM control structure (VMCS). */
-        RTR0MEMOBJ                  pMemObjVMCS;
         /** Physical address of the VM control structure (VMCS). */
         RTHCPHYS                    pVMCSPhys;
+        /** R0 memory object for the VM control structure (VMCS). */
+        RTR0MEMOBJ                  pMemObjVMCS;
         /** Virtual address of the VM control structure (VMCS). */
         R0PTRTYPE(void *)           pVMCS;
 
         /** Ring 0 handlers for VT-x. */
-        DECLR0CALLBACKMEMBER(int,  pfnStartVM,(RTHCUINT fResume, PCPUMCTX pCtx, PVMCSCACHE pCache, PVM pVM, PVMCPU pVCpu));
+        PFNHWACCMVMXSTARTVM         pfnStartVM;
 
         /** Current VMX_VMCS_CTRL_PROC_EXEC_CONTROLS. */
         uint64_t                    proc_ctls;
 
+        /** Current VMX_VMCS_CTRL_PROC_EXEC2_CONTROLS. */
+        uint64_t                    proc_ctls2;
+
+        /** Physical address of the virtual APIC page for TPR caching. */
+        RTHCPHYS                    pVAPICPhys;
+        /** R0 memory object for the virtual APIC page for TPR caching. */
+        RTR0MEMOBJ                  pMemObjVAPIC;
+        /** Virtual address of the virtual APIC page for TPR caching. */
+        R0PTRTYPE(uint8_t *)        pVAPIC;
+
         /** Current CR0 mask. */
         uint64_t                    cr0_mask;
         /** Current CR4 mask. */
@@ -499,6 +592,36 @@ typedef struct HWACCMCPU
         /** Current EPTP. */
         RTHCPHYS                    GCPhysEPTP;
 
+        /** Physical address of the MSR bitmap (1 page). */
+        RTHCPHYS                    pMSRBitmapPhys;
+        /** R0 memory object for the MSR bitmap (1 page). */
+        RTR0MEMOBJ                  pMemObjMSRBitmap;
+        /** Virtual address of the MSR bitmap (1 page). */
+        R0PTRTYPE(uint8_t *)        pMSRBitmap;
+
+#ifdef VBOX_WITH_AUTO_MSR_LOAD_RESTORE
+        /** Physical address of the guest MSR load area (1 page). */
+        RTHCPHYS                    pGuestMSRPhys;
+        /** R0 memory object for the guest MSR load area (1 page). */
+        RTR0MEMOBJ                  pMemObjGuestMSR;
+        /** Virtual address of the guest MSR load area (1 page). */
+        R0PTRTYPE(uint8_t *)        pGuestMSR;
+
+        /** Physical address of the MSR load area (1 page). */
+        RTHCPHYS                    pHostMSRPhys;
+        /** R0 memory object for the MSR load area (1 page). */
+        RTR0MEMOBJ                  pMemObjHostMSR;
+        /** Virtual address of the MSR load area (1 page). */
+        R0PTRTYPE(uint8_t *)        pHostMSR;
+#endif /* VBOX_WITH_AUTO_MSR_LOAD_RESTORE */
+
+        /* Number of automatically loaded/restored MSRs. */
+        uint32_t                    cCachedMSRs;
+        uint32_t                    uAlignement;
+
+        /* Last use TSC offset value. (cached) */
+        uint64_t                    u64TSCOffset;
+
         /** VMCS cache. */
         VMCSCACHE                   VMCSCache;
 
@@ -530,6 +653,13 @@ typedef struct HWACCMCPU
 
     struct
     {
+        /** R0 memory object for the host VM control block (VMCB). */
+        RTR0MEMOBJ                  pMemObjVMCBHost;
+        /** Physical address of the host VM control block (VMCB). */
+        RTHCPHYS                    pVMCBHostPhys;
+        /** Virtual address of the host VM control block (VMCB). */
+        R0PTRTYPE(void *)           pVMCBHost;
+
         /** R0 memory object for the VM control block (VMCB). */
         RTR0MEMOBJ                  pMemObjVMCB;
         /** Physical address of the VM control block (VMCB). */
@@ -538,8 +668,14 @@ typedef struct HWACCMCPU
         R0PTRTYPE(void *)           pVMCB;
 
         /** Ring 0 handlers for VT-x. */
-        DECLR0CALLBACKMEMBER(int, pfnVMRun,(RTHCPHYS pVMCBHostPhys, RTHCPHYS pVMCBPhys, PCPUMCTX pCtx, PVM pVM, PVMCPU pVCpu));
+        PFNHWACCMSVMVMRUN           pfnVMRun;
 
+        /** R0 memory object for the MSR bitmap (8kb). */
+        RTR0MEMOBJ                  pMemObjMSRBitmap;
+        /** Physical address of the MSR bitmap (8kb). */
+        RTHCPHYS                    pMSRBitmapPhys;
+        /** Virtual address of the MSR bitmap. */
+        R0PTRTYPE(void *)           pMSRBitmap;
     } svm;
 
     /** Event injection state. */
@@ -562,6 +698,25 @@ typedef struct HWACCMCPU
         uint64_t                cr0;
     } EmulateIoBlock;
 
+    struct
+    {
+        /* Pending IO operation type. */
+        HWACCMPENDINGIO         enmType;
+        uint32_t                uPadding;
+        RTGCPTR                 GCPtrRip;
+        RTGCPTR                 GCPtrRipNext;
+        union
+        {
+            struct
+            {
+                unsigned        uPort;
+                unsigned        uAndVal;
+                unsigned        cbSize;
+            } Port;
+            uint64_t            aRaw[2];
+        } s;
+    } PendingIO;
+
     /** Currenty shadow paging mode. */
     PGMMODE                 enmShadowMode;
 
@@ -569,6 +724,26 @@ typedef struct HWACCMCPU
      * HWACCMR0Enter and cleared in HWACCMR0Leave. */
     RTCPUID                 idEnteredCpu;
 
+    /** To keep track of pending TLB shootdown pages. (SMP guest only) */
+    struct
+    {
+        RTGCPTR             aPages[HWACCM_MAX_TLB_SHOOTDOWN_PAGES];
+        unsigned            cPages;
+    } TlbShootdown;
+
+    /** For saving stack space, the disassembler state is allocated here instead of
+     * on the stack.
+     * @note The DISCPUSTATE structure is not R3/R0/RZ clean!  */
+    union
+    {
+        /** The disassembler scratch space. */
+        DISCPUSTATE         DisState;
+        /** Padding. */
+        uint8_t             abDisStatePadding[DISCPUSTATE_PADDING_SIZE];
+    };
+
+    RTUINT                  padding2[1];
+
     STAMPROFILEADV          StatEntry;
     STAMPROFILEADV          StatExit1;
     STAMPROFILEADV          StatExit2;
@@ -611,8 +786,11 @@ typedef struct HWACCMCPU
     STAMCOUNTER             StatExitCRxRead[16];
     STAMCOUNTER             StatExitDRxWrite;
     STAMCOUNTER             StatExitDRxRead;
+    STAMCOUNTER             StatExitRdmsr;
+    STAMCOUNTER             StatExitWrmsr;
     STAMCOUNTER             StatExitCLTS;
     STAMCOUNTER             StatExitHlt;
+    STAMCOUNTER             StatExitMwait;
     STAMCOUNTER             StatExitLMSW;
     STAMCOUNTER             StatExitIOWrite;
     STAMCOUNTER             StatExitIORead;
@@ -620,6 +798,7 @@ typedef struct HWACCMCPU
     STAMCOUNTER             StatExitIOStringRead;
     STAMCOUNTER             StatExitIrqWindow;
     STAMCOUNTER             StatExitMaxResume;
+    STAMCOUNTER             StatExitPreemptPending;
     STAMCOUNTER             StatIntReinject;
     STAMCOUNTER             StatPendingHostIrq;
 
@@ -632,18 +811,22 @@ typedef struct HWACCMCPU
     STAMCOUNTER             StatFlushTLBCRxChange;
     STAMCOUNTER             StatFlushASID;
     STAMCOUNTER             StatFlushTLBInvlpga;
+    STAMCOUNTER             StatTlbShootdown;
+    STAMCOUNTER             StatTlbShootdownFlush;
 
     STAMCOUNTER             StatSwitchGuestIrq;
     STAMCOUNTER             StatSwitchToR3;
 
     STAMCOUNTER             StatTSCOffset;
     STAMCOUNTER             StatTSCIntercept;
+    STAMCOUNTER             StatTSCInterceptOverFlow;
 
     STAMCOUNTER             StatExitReasonNPF;
     STAMCOUNTER             StatDRxArmed;
     STAMCOUNTER             StatDRxContextSwitch;
     STAMCOUNTER             StatDRxIOCheck;
 
+
 #ifdef VBOX_WITH_STATISTICS
     R3PTRTYPE(PSTAMCOUNTER) paStatExitReason;
     R0PTRTYPE(PSTAMCOUNTER) paStatExitReasonR0;
@@ -662,11 +845,11 @@ VMMR0DECL(PHWACCM_CPUINFO) HWACCMR0GetCurrentCpuEx(RTCPUID idCpu);
 
 
 #ifdef VBOX_STRICT
-VMMR0DECL(void) HWACCMDumpRegs(PVM pVM, PCPUMCTX pCtx);
-VMMR0DECL(void) HWACCMR0DumpDescriptor(PX86DESCHC  Desc, RTSEL Sel, const char *pszMsg);
+VMMR0DECL(void) HWACCMDumpRegs(PVM pVM, PVMCPU pVCpu, PCPUMCTX pCtx);
+VMMR0DECL(void) HWACCMR0DumpDescriptor(PCX86DESCHC pDesc, RTSEL Sel, const char *pszMsg);
 #else
-#define HWACCMDumpRegs(a, b)                do { } while (0)
-#define HWACCMR0DumpDescriptor(a, b, c)     do { } while (0)
+# define HWACCMDumpRegs(a, b ,c)            do { } while (0)
+# define HWACCMR0DumpDescriptor(a, b, c)    do { } while (0)
 #endif
 
 /* Dummy callback handlers. */
@@ -681,6 +864,10 @@ VMMR0DECL(int) HWACCMR0DummyRunGuestCode(PVM pVM, PVMCPU pVCpu, PCPUMCTX pCtx);
 VMMR0DECL(int) HWACCMR0DummySaveHostState(PVM pVM, PVMCPU pVCpu);
 VMMR0DECL(int) HWACCMR0DummyLoadGuestState(PVM pVM, PVMCPU pVCpu, PCPUMCTX pCtx);
 
+# ifdef VBOX_WITH_KERNEL_USING_XMM
+DECLASM(int)   hwaccmR0VMXStartVMWrapXMM(RTHCUINT fResume, PCPUMCTX pCtx, PVMCSCACHE pCache, PVM pVM, PVMCPU pVCpu, PFNHWACCMVMXSTARTVM pfnStartVM);
+DECLASM(int)   hwaccmR0SVMRunWrapXMM(RTHCPHYS pVMCBHostPhys, RTHCPHYS pVMCBPhys, PCPUMCTX pCtx, PVM pVM, PVMCPU pVCpu, PFNHWACCMSVMVMRUN pfnVMRun);
+# endif
 
 # ifdef VBOX_WITH_HYBRID_32BIT_KERNEL
 /**
@@ -701,7 +888,7 @@ DECLASM(uint64_t) hwaccmR0Get64bitCR3(void);
 
 /** @} */
 
-__END_DECLS
+RT_C_DECLS_END
 
 #endif
 
