@@ -1,4 +1,4 @@
-/* $Id: VMReq.cpp 13858 2008-11-05 13:45:41Z vboxsync $ */
+/* $Id: VMReq.cpp $ */
 /** @file
  * VM - Virtual Machine
  */
@@ -60,7 +60,9 @@ static int  vmR3ReqProcessOneU(PUVM pUVM, PVMREQ pReq);
  * @returns VERR_TIMEOUT if cMillies was reached without the packet being completed.
  *
  * @param   pVM             The VM handle.
- * @param   enmDest         Destination of the request packet (global or per VCPU).
+ * @param   idDstCpu        The destination CPU(s). Either a specific CPU ID or
+ *                          one of the following special values:
+ *                              VMCPUID_ANY, VMCPUID_ANY_QUEUE, VMCPUID_ALL or VMCPUID_ALL_REVERSE.
  * @param   ppReq           Where to store the pointer to the request.
  *                          This will be NULL or a valid request pointer not matter what happends.
  * @param   cMillies        Number of milliseconds to wait for the request to
@@ -71,11 +73,11 @@ static int  vmR3ReqProcessOneU(PUVM pUVM, PVMREQ pReq);
  *                          Not possible to pass 64-bit arguments!
  * @param   ...             Function arguments.
  */
-VMMR3DECL(int) VMR3ReqCall(PVM pVM, VMREQDEST enmDest, PVMREQ *ppReq, unsigned cMillies, PFNRT pfnFunction, unsigned cArgs, ...)
+VMMR3DECL(int) VMR3ReqCall(PVM pVM, VMCPUID idDstCpu, PVMREQ *ppReq, unsigned cMillies, PFNRT pfnFunction, unsigned cArgs, ...)
 {
     va_list va;
     va_start(va, cArgs);
-    int rc = VMR3ReqCallVU(pVM->pUVM, enmDest, ppReq, cMillies, VMREQFLAGS_VBOX_STATUS, pfnFunction, cArgs, va);
+    int rc = VMR3ReqCallVU(pVM->pUVM, idDstCpu, ppReq, cMillies, VMREQFLAGS_VBOX_STATUS, pfnFunction, cArgs, va);
     va_end(va);
     return rc;
 }
@@ -94,7 +96,9 @@ VMMR3DECL(int) VMR3ReqCall(PVM pVM, VMREQDEST enmDest, PVMREQ *ppReq, unsigned c
  * @returns VERR_TIMEOUT if cMillies was reached without the packet being completed.
  *
  * @param   pUVM            Pointer to the user mode VM structure.
- * @param   enmDest         Destination of the request packet (global or per VCPU).
+ * @param   idDstCpu        The destination CPU(s). Either a specific CPU ID or
+ *                          one of the following special values:
+ *                              VMCPUID_ANY, VMCPUID_ANY_QUEUE, VMCPUID_ALL or VMCPUID_ALL_REVERSE.
  * @param   ppReq           Where to store the pointer to the request.
  *                          This will be NULL or a valid request pointer not matter what happends.
  * @param   cMillies        Number of milliseconds to wait for the request to
@@ -105,11 +109,11 @@ VMMR3DECL(int) VMR3ReqCall(PVM pVM, VMREQDEST enmDest, PVMREQ *ppReq, unsigned c
  *                          Not possible to pass 64-bit arguments!
  * @param   ...             Function arguments.
  */
-VMMR3DECL(int) VMR3ReqCallVoidU(PUVM pUVM, VMREQDEST enmDest, PVMREQ *ppReq, unsigned cMillies, PFNRT pfnFunction, unsigned cArgs, ...)
+VMMR3DECL(int) VMR3ReqCallVoidU(PUVM pUVM, VMCPUID idDstCpu, PVMREQ *ppReq, unsigned cMillies, PFNRT pfnFunction, unsigned cArgs, ...)
 {
     va_list va;
     va_start(va, cArgs);
-    int rc = VMR3ReqCallVU(pUVM, enmDest, ppReq, cMillies, VMREQFLAGS_VOID, pfnFunction, cArgs, va);
+    int rc = VMR3ReqCallVU(pUVM, idDstCpu, ppReq, cMillies, VMREQFLAGS_VOID, pfnFunction, cArgs, va);
     va_end(va);
     return rc;
 }
@@ -128,7 +132,9 @@ VMMR3DECL(int) VMR3ReqCallVoidU(PUVM pUVM, VMREQDEST enmDest, PVMREQ *ppReq, uns
  * @returns VERR_TIMEOUT if cMillies was reached without the packet being completed.
  *
  * @param   pVM             The VM handle.
- * @param   enmDest         Destination of the request packet (global or per VCPU).
+ * @param   idDstCpu        The destination CPU(s). Either a specific CPU ID or
+ *                          one of the following special values:
+ *                              VMCPUID_ANY, VMCPUID_ANY_QUEUE, VMCPUID_ALL or VMCPUID_ALL_REVERSE.
  * @param   ppReq           Where to store the pointer to the request.
  *                          This will be NULL or a valid request pointer not matter what happends.
  * @param   cMillies        Number of milliseconds to wait for the request to
@@ -139,11 +145,11 @@ VMMR3DECL(int) VMR3ReqCallVoidU(PUVM pUVM, VMREQDEST enmDest, PVMREQ *ppReq, uns
  *                          Not possible to pass 64-bit arguments!
  * @param   ...             Function arguments.
  */
-VMMR3DECL(int) VMR3ReqCallVoid(PVM pVM, VMREQDEST enmDest, PVMREQ *ppReq, unsigned cMillies, PFNRT pfnFunction, unsigned cArgs, ...)
+VMMR3DECL(int) VMR3ReqCallVoid(PVM pVM, VMCPUID idDstCpu, PVMREQ *ppReq, unsigned cMillies, PFNRT pfnFunction, unsigned cArgs, ...)
 {
     va_list va;
     va_start(va, cArgs);
-    int rc = VMR3ReqCallVU(pVM->pUVM, enmDest, ppReq, cMillies, VMREQFLAGS_VOID, pfnFunction, cArgs, va);
+    int rc = VMR3ReqCallVU(pVM->pUVM, idDstCpu, ppReq, cMillies, VMREQFLAGS_VOID, pfnFunction, cArgs, va);
     va_end(va);
     return rc;
 }
@@ -162,7 +168,9 @@ VMMR3DECL(int) VMR3ReqCallVoid(PVM pVM, VMREQDEST enmDest, PVMREQ *ppReq, unsign
  * @returns VERR_TIMEOUT if cMillies was reached without the packet being completed.
  *
  * @param   pVM             The VM handle.
- * @param   enmDest         Destination of the request packet (global or per VCPU).
+ * @param   idDstCpu        The destination CPU(s). Either a specific CPU ID or
+ *                          one of the following special values:
+ *                              VMCPUID_ANY, VMCPUID_ANY_QUEUE, VMCPUID_ALL or VMCPUID_ALL_REVERSE.
  * @param   ppReq           Where to store the pointer to the request.
  *                          This will be NULL or a valid request pointer not matter what happends, unless fFlags
  *                          contains VMREQFLAGS_NO_WAIT when it will be optional and always NULL.
@@ -175,11 +183,11 @@ VMMR3DECL(int) VMR3ReqCallVoid(PVM pVM, VMREQDEST enmDest, PVMREQ *ppReq, unsign
  *                          Not possible to pass 64-bit arguments!
  * @param   ...             Function arguments.
  */
-VMMR3DECL(int) VMR3ReqCallEx(PVM pVM, VMREQDEST enmDest, PVMREQ *ppReq, unsigned cMillies, unsigned fFlags, PFNRT pfnFunction, unsigned cArgs, ...)
+VMMR3DECL(int) VMR3ReqCallEx(PVM pVM, VMCPUID idDstCpu, PVMREQ *ppReq, unsigned cMillies, unsigned fFlags, PFNRT pfnFunction, unsigned cArgs, ...)
 {
     va_list va;
     va_start(va, cArgs);
-    int rc = VMR3ReqCallVU(pVM->pUVM, enmDest, ppReq, cMillies, fFlags, pfnFunction, cArgs, va);
+    int rc = VMR3ReqCallVU(pVM->pUVM, idDstCpu, ppReq, cMillies, fFlags, pfnFunction, cArgs, va);
     va_end(va);
     return rc;
 }
@@ -198,7 +206,9 @@ VMMR3DECL(int) VMR3ReqCallEx(PVM pVM, VMREQDEST enmDest, PVMREQ *ppReq, unsigned
  * @returns VERR_TIMEOUT if cMillies was reached without the packet being completed.
  *
  * @param   pUVM            Pointer to the user mode VM structure.
- * @param   enmDest         Destination of the request packet (global or per VCPU).
+ * @param   idDstCpu        The destination CPU(s). Either a specific CPU ID or
+ *                          one of the following special values:
+ *                              VMCPUID_ANY, VMCPUID_ANY_QUEUE, VMCPUID_ALL or VMCPUID_ALL_REVERSE.
  * @param   ppReq           Where to store the pointer to the request.
  *                          This will be NULL or a valid request pointer not matter what happends, unless fFlags
  *                          contains VMREQFLAGS_NO_WAIT when it will be optional and always NULL.
@@ -211,11 +221,11 @@ VMMR3DECL(int) VMR3ReqCallEx(PVM pVM, VMREQDEST enmDest, PVMREQ *ppReq, unsigned
  *                          Not possible to pass 64-bit arguments!
  * @param   ...             Function arguments.
  */
-VMMR3DECL(int) VMR3ReqCallU(PUVM pUVM, VMREQDEST enmDest, PVMREQ *ppReq, unsigned cMillies, unsigned fFlags, PFNRT pfnFunction, unsigned cArgs, ...)
+VMMR3DECL(int) VMR3ReqCallU(PUVM pUVM, VMCPUID idDstCpu, PVMREQ *ppReq, unsigned cMillies, unsigned fFlags, PFNRT pfnFunction, unsigned cArgs, ...)
 {
     va_list va;
     va_start(va, cArgs);
-    int rc = VMR3ReqCallVU(pUVM, enmDest, ppReq, cMillies, fFlags, pfnFunction, cArgs, va);
+    int rc = VMR3ReqCallVU(pUVM, idDstCpu, ppReq, cMillies, fFlags, pfnFunction, cArgs, va);
     va_end(va);
     return rc;
 }
@@ -234,7 +244,9 @@ VMMR3DECL(int) VMR3ReqCallU(PUVM pUVM, VMREQDEST enmDest, PVMREQ *ppReq, unsigne
  * @returns VERR_TIMEOUT if cMillies was reached without the packet being completed.
  *
  * @param   pUVM            Pointer to the user mode VM structure.
- * @param   enmDest         Destination of the request packet (global or per VCPU).
+ * @param   idDstCpu        The destination CPU(s). Either a specific CPU ID or
+ *                          one of the following special values:
+ *                              VMCPUID_ANY, VMCPUID_ANY_QUEUE, VMCPUID_ALL or VMCPUID_ALL_REVERSE.
  * @param   ppReq           Where to store the pointer to the request.
  *                          This will be NULL or a valid request pointer not matter what happends, unless fFlags
  *                          contains VMREQFLAGS_NO_WAIT when it will be optional and always NULL.
@@ -247,16 +259,16 @@ VMMR3DECL(int) VMR3ReqCallU(PUVM pUVM, VMREQDEST enmDest, PVMREQ *ppReq, unsigne
  *                          Stuff which differs in size from uintptr_t is gonna make trouble, so don't try!
  * @param   Args            Argument vector.
  */
-VMMR3DECL(int) VMR3ReqCallVU(PUVM pUVM, VMREQDEST enmDest, PVMREQ *ppReq, unsigned cMillies, unsigned fFlags, PFNRT pfnFunction, unsigned cArgs, va_list Args)
+VMMR3DECL(int) VMR3ReqCallVU(PUVM pUVM, VMCPUID idDstCpu, PVMREQ *ppReq, unsigned cMillies, unsigned fFlags, PFNRT pfnFunction, unsigned cArgs, va_list Args)
 {
-    LogFlow(("VMR3ReqCallV: cMillies=%d fFlags=%#x pfnFunction=%p cArgs=%d\n", cMillies, fFlags, pfnFunction, cArgs));
+    LogFlow(("VMR3ReqCallV: idDstCpu=%u cMillies=%d fFlags=%#x pfnFunction=%p cArgs=%d\n", idDstCpu, cMillies, fFlags, pfnFunction, cArgs));
 
     /*
      * Validate input.
      */
     AssertPtrReturn(pfnFunction, VERR_INVALID_POINTER);
     AssertPtrReturn(pUVM, VERR_INVALID_POINTER);
-    AssertReturn(!(fFlags & ~(VMREQFLAGS_RETURN_MASK | VMREQFLAGS_NO_WAIT)), VERR_INVALID_PARAMETER);
+    AssertReturn(!(fFlags & ~(VMREQFLAGS_RETURN_MASK | VMREQFLAGS_NO_WAIT | VMREQFLAGS_POKE)), VERR_INVALID_PARAMETER);
     if (!(fFlags & VMREQFLAGS_NO_WAIT) || ppReq)
     {
         AssertPtrReturn(ppReq, VERR_INVALID_POINTER);
@@ -270,7 +282,7 @@ VMMR3DECL(int) VMR3ReqCallVU(PUVM pUVM, VMREQDEST enmDest, PVMREQ *ppReq, unsign
     /*
      * Allocate request
      */
-    int rc = VMR3ReqAllocU(pUVM, &pReq, VMREQTYPE_INTERNAL, enmDest);
+    int rc = VMR3ReqAllocU(pUVM, &pReq, VMREQTYPE_INTERNAL, idDstCpu);
     if (RT_FAILURE(rc))
         return rc;
 
@@ -318,10 +330,12 @@ static void vmr3ReqJoinFreeSub(volatile PVMREQ *ppHead, PVMREQ pList)
         PVMREQ pTail = pHead;
         while (pTail->pNext)
             pTail = pTail->pNext;
-        pTail->pNext = pList;
+        ASMAtomicWritePtr((void * volatile *)&pTail->pNext, pList);
+        ASMCompilerBarrier();
         if (ASMAtomicCmpXchgPtr((void * volatile *)ppHead, (void *)pHead, pList))
             return;
-        pTail->pNext = NULL;
+        ASMAtomicWritePtr((void * volatile *)&pTail->pNext, NULL);
+        ASMCompilerBarrier();
         if (ASMAtomicCmpXchgPtr((void * volatile *)ppHead, (void *)pHead, NULL))
             return;
         pList = pHead;
@@ -369,11 +383,13 @@ static void vmr3ReqJoinFree(PVMINTUSERPERVM pVMInt, PVMREQ pList)
  * @param   pVM             VM handle.
  * @param   ppReq           Where to store the pointer to the allocated packet.
  * @param   enmType         Package type.
- * @param   enmDest         Destination of the request packet (global or per VCPU).
+ * @param   idDstCpu        The destination CPU(s). Either a specific CPU ID or
+ *                          one of the following special values:
+ *                              VMCPUID_ANY, VMCPUID_ANY_QUEUE, VMCPUID_ALL or VMCPUID_ALL_REVERSE.
  */
-VMMR3DECL(int) VMR3ReqAlloc(PVM pVM, PVMREQ *ppReq, VMREQTYPE enmType, VMREQDEST enmDest)
+VMMR3DECL(int) VMR3ReqAlloc(PVM pVM, PVMREQ *ppReq, VMREQTYPE enmType, VMCPUID idDstCpu)
 {
-    return VMR3ReqAllocU(pVM->pUVM, ppReq, enmType, enmDest);
+    return VMR3ReqAllocU(pVM->pUVM, ppReq, enmType, idDstCpu);
 }
 
 
@@ -388,9 +404,11 @@ VMMR3DECL(int) VMR3ReqAlloc(PVM pVM, PVMREQ *ppReq, VMREQTYPE enmType, VMREQDEST
  * @param   pUVM            Pointer to the user mode VM structure.
  * @param   ppReq           Where to store the pointer to the allocated packet.
  * @param   enmType         Package type.
- * @param   enmDest         Destination of the request packet (global or per VCPU).
+ * @param   idDstCpu        The destination CPU(s). Either a specific CPU ID or
+ *                          one of the following special values:
+ *                              VMCPUID_ANY, VMCPUID_ANY_QUEUE, VMCPUID_ALL or VMCPUID_ALL_REVERSE.
  */
-VMMR3DECL(int) VMR3ReqAllocU(PUVM pUVM, PVMREQ *ppReq, VMREQTYPE enmType, VMREQDEST enmDest)
+VMMR3DECL(int) VMR3ReqAllocU(PUVM pUVM, PVMREQ *ppReq, VMREQTYPE enmType, VMCPUID idDstCpu)
 {
     /*
      * Validate input.
@@ -400,7 +418,12 @@ VMMR3DECL(int) VMR3ReqAllocU(PUVM pUVM, PVMREQ *ppReq, VMREQTYPE enmType, VMREQD
                      enmType, VMREQTYPE_INVALID + 1, VMREQTYPE_MAX - 1),
                     VERR_VM_REQUEST_INVALID_TYPE);
     AssertPtrReturn(ppReq, VERR_INVALID_POINTER);
-    AssertMsgReturn(enmDest == VMREQDEST_ANY || enmDest == VMREQDEST_BROADCAST || (unsigned)enmDest < pUVM->pVM->cCPUs, ("Invalid destination %d (max=%d)\n", enmDest, pUVM->pVM->cCPUs), VERR_INVALID_PARAMETER);
+    AssertMsgReturn(    idDstCpu == VMCPUID_ANY
+                    ||  idDstCpu == VMCPUID_ANY_QUEUE
+                    ||  idDstCpu < pUVM->cCpus
+                    ||  idDstCpu == VMCPUID_ALL
+                    ||  idDstCpu == VMCPUID_ALL_REVERSE,
+                    ("Invalid destination %u (max=%u)\n", idDstCpu, pUVM->cCpus), VERR_INVALID_PARAMETER);
 
     /*
      * Try get a recycled packet.
@@ -471,7 +494,7 @@ VMMR3DECL(int) VMR3ReqAllocU(PUVM pUVM, PVMREQ *ppReq, VMREQTYPE enmType, VMREQD
             pReq->iStatus  = VERR_VM_REQUEST_STATUS_STILL_PENDING;
             pReq->fFlags   = VMREQFLAGS_VBOX_STATUS;
             pReq->enmType  = enmType;
-            pReq->enmDest  = enmDest;
+            pReq->idDstCpu = idDstCpu;
 
             *ppReq = pReq;
             STAM_COUNTER_INC(&pUVM->vm.s.StatReqAllocRecycled);
@@ -508,7 +531,7 @@ VMMR3DECL(int) VMR3ReqAllocU(PUVM pUVM, PVMREQ *ppReq, VMREQTYPE enmType, VMREQD
     pReq->fEventSemClear = true;
     pReq->fFlags   = VMREQFLAGS_VBOX_STATUS;
     pReq->enmType  = enmType;
-    pReq->enmDest  = enmDest;
+    pReq->idDstCpu = idDstCpu;
 
     *ppReq = pReq;
     STAM_COUNTER_INC(&pUVM->vm.s.StatReqAllocNew);
@@ -563,8 +586,9 @@ VMMR3DECL(int) VMR3ReqFree(PVMREQ pReq)
         PVMREQ pNext;
         do
         {
-            pNext = *ppHead;
-            ASMAtomicXchgPtr((void * volatile *)&pReq->pNext, pNext);
+            pNext = (PVMREQ)ASMAtomicUoReadPtr((void * volatile *)ppHead);
+            ASMAtomicWritePtr((void * volatile *)&pReq->pNext, pNext);
+            ASMCompilerBarrier();
         } while (!ASMAtomicCmpXchgPtr((void * volatile *)ppHead, (void *)pReq, (void *)pNext));
     }
     else
@@ -612,6 +636,7 @@ VMMR3DECL(int) VMR3ReqQueue(PVMREQ pReq, unsigned cMillies)
                     ("Invalid package type %d valid range %d-%d inclusivly. This was verified on alloc too...\n",
                      pReq->enmType, VMREQTYPE_INVALID + 1, VMREQTYPE_MAX - 1),
                     VERR_VM_REQUEST_INVALID_TYPE);
+    Assert(!(pReq->fFlags & ~(VMREQFLAGS_RETURN_MASK | VMREQFLAGS_NO_WAIT | VMREQFLAGS_POKE)));
 
     /*
      * Are we the EMT or not?
@@ -619,62 +644,48 @@ VMMR3DECL(int) VMR3ReqQueue(PVMREQ pReq, unsigned cMillies)
      */
     int     rc      = VINF_SUCCESS;
     PUVM    pUVM    = ((VMREQ volatile *)pReq)->pUVM;                 /* volatile paranoia */
-    PUVMCPU pUVMCPU = (PUVMCPU)RTTlsGet(pUVM->vm.s.idxTLS);
+    PUVMCPU pUVCpu  = (PUVMCPU)RTTlsGet(pUVM->vm.s.idxTLS);
 
-    if (pReq->enmDest == VMREQDEST_BROADCAST)
+    if (pReq->idDstCpu == VMCPUID_ALL)
     {
-        unsigned fFlags = ((VMREQ volatile *)pReq)->fFlags;     /* volatile paranoia */
-
-        for (unsigned i=0;i<pUVM->pVM->cCPUs;i++)
+        /* One-by-one. */
+        Assert(!(pReq->fFlags & VMREQFLAGS_NO_WAIT));
+        for (unsigned i = 0; i < pUVM->cCpus; i++)
         {
-            if (   !pUVMCPU
-                ||  pUVMCPU->idCpu != i)
-            {
-                /*
-                 * Insert it.
-                 */
-                pReq->enmState = VMREQSTATE_QUEUED;
-                PVMREQ pNext;
-                do
-                {
-                    pNext = pUVM->aCpus[i].vm.s.pReqs;
-                    pReq->pNext = pNext;
-                } while (!ASMAtomicCmpXchgPtr((void * volatile *)&pUVM->aCpus[i].vm.s.pReqs, (void *)pReq, (void *)pNext));
-
-                /*
-                 * Notify EMT.
-                 */
-                if (pUVM->pVM)
-                    VMCPU_FF_SET(pUVM->pVM, VM_FF_REQUEST, i);
-                /* @todo: VMR3NotifyFFU*/
-                AssertFailed();
-                VMR3NotifyFFU(pUVM, false);
-
-                /*
-                 * Wait and return.
-                 */
-                if (!(fFlags & VMREQFLAGS_NO_WAIT))
-                    rc = VMR3ReqWait(pReq, cMillies);
-                LogFlow(("VMR3ReqQueue: returns %Rrc\n", rc));
-            }
-            else
-            {
-                /*
-                 * The requester was EMT, just execute it.
-                 */
-                pReq->enmState = VMREQSTATE_QUEUED;
-                rc = vmR3ReqProcessOneU(pUVM, pReq);
-                LogFlow(("VMR3ReqQueue: returns %Rrc (processed)\n", rc));
-            }
-        } /* for each VMCPU */
+            /* Reinit some members. */
+            pReq->enmState = VMREQSTATE_ALLOCATED;
+            pReq->idDstCpu = i;
+            rc = VMR3ReqQueue(pReq, cMillies);
+            if (RT_FAILURE(rc))
+                break;
+        }
     }
-    else
-    if (    pReq->enmDest  != VMREQDEST_ANY  /* for a specific VMCPU? */
-        &&  pUVMCPU->idCpu != (unsigned)pReq->enmDest)
+    else if (pReq->idDstCpu == VMCPUID_ALL_REVERSE)
     {
-        RTCPUID  idTarget = (RTCPUID)pReq->enmDest;
+        /* One-by-one. */
+        Assert(!(pReq->fFlags & VMREQFLAGS_NO_WAIT));
+        for (int i = pUVM->cCpus-1; i >= 0; i--)
+        {
+            /* Reinit some members. */
+            pReq->enmState = VMREQSTATE_ALLOCATED;
+            pReq->idDstCpu = i;
+            rc = VMR3ReqQueue(pReq, cMillies);
+            if (RT_FAILURE(rc))
+                break;
+        }
+    }
+    else if (   pReq->idDstCpu != VMCPUID_ANY   /* for a specific VMCPU? */
+             && pReq->idDstCpu != VMCPUID_ANY_QUEUE 
+             && (   !pUVCpu                     /* and it's not the current thread. */
+                 || pUVCpu->idCpu != pReq->idDstCpu))
+    {
+        VMCPUID  idTarget = pReq->idDstCpu;     Assert(idTarget < pUVM->cCpus);
+        PVMCPU   pVCpu = &pUVM->pVM->aCpus[idTarget];
         unsigned fFlags = ((VMREQ volatile *)pReq)->fFlags;     /* volatile paranoia */
 
+        /* Fetch the right UVMCPU */
+        pUVCpu = &pUVM->aCpus[idTarget];
+
         /*
          * Insert it.
          */
@@ -682,18 +693,17 @@ VMMR3DECL(int) VMR3ReqQueue(PVMREQ pReq, unsigned cMillies)
         PVMREQ pNext;
         do
         {
-            pNext = pUVM->aCpus[idTarget].vm.s.pReqs;
-            pReq->pNext = pNext;
-        } while (!ASMAtomicCmpXchgPtr((void * volatile *)&pUVM->aCpus[idTarget].vm.s.pReqs, (void *)pReq, (void *)pNext));
+            pNext = (PVMREQ)ASMAtomicUoReadPtr((void * volatile *)&pUVCpu->vm.s.pReqs);
+            ASMAtomicWritePtr((void * volatile *)&pReq->pNext, pNext);
+            ASMCompilerBarrier();
+        } while (!ASMAtomicCmpXchgPtr((void * volatile *)&pUVCpu->vm.s.pReqs, (void *)pReq, (void *)pNext));
 
         /*
          * Notify EMT.
          */
         if (pUVM->pVM)
-            VMCPU_FF_SET(pUVM->pVM, VM_FF_REQUEST, idTarget);
-        /* @todo: VMR3NotifyFFU*/
-        AssertFailed();
-        VMR3NotifyFFU(pUVM, false);
+            VMCPU_FF_SET(pVCpu, VMCPU_FF_REQUEST);
+        VMR3NotifyCpuFFU(pUVCpu, fFlags & VMREQFLAGS_POKE ? VMNOTIFYFF_FLAGS_POKE : 0);
 
         /*
          * Wait and return.
@@ -702,11 +712,14 @@ VMMR3DECL(int) VMR3ReqQueue(PVMREQ pReq, unsigned cMillies)
             rc = VMR3ReqWait(pReq, cMillies);
         LogFlow(("VMR3ReqQueue: returns %Rrc\n", rc));
     }
-    else if (    pReq->enmDest == VMREQDEST_ANY
-             &&  !pUVMCPU /* only EMT threads have a valid pointer stored in the TLS slot. */)
+    else if (   (    pReq->idDstCpu == VMCPUID_ANY
+                 && !pUVCpu /* only EMT threads have a valid pointer stored in the TLS slot. */)
+             || pReq->idDstCpu == VMCPUID_ANY_QUEUE)
     {
         unsigned fFlags = ((VMREQ volatile *)pReq)->fFlags;     /* volatile paranoia */
 
+        Assert(pReq->idDstCpu != VMCPUID_ANY_QUEUE || pUVCpu);
+
         /*
          * Insert it.
          */
@@ -714,8 +727,9 @@ VMMR3DECL(int) VMR3ReqQueue(PVMREQ pReq, unsigned cMillies)
         PVMREQ pNext;
         do
         {
-            pNext = pUVM->vm.s.pReqs;
-            pReq->pNext = pNext;
+            pNext = (PVMREQ)ASMAtomicUoReadPtr((void * volatile *)&pUVM->vm.s.pReqs);
+            ASMAtomicWritePtr((void * volatile *)&pReq->pNext, pNext);
+            ASMCompilerBarrier();
         } while (!ASMAtomicCmpXchgPtr((void * volatile *)&pUVM->vm.s.pReqs, (void *)pReq, (void *)pNext));
 
         /*
@@ -723,7 +737,7 @@ VMMR3DECL(int) VMR3ReqQueue(PVMREQ pReq, unsigned cMillies)
          */
         if (pUVM->pVM)
             VM_FF_SET(pUVM->pVM, VM_FF_REQUEST);
-        VMR3NotifyFFU(pUVM, false);
+        VMR3NotifyGlobalFFU(pUVM, fFlags & VMREQFLAGS_POKE ? VMNOTIFYFF_FLAGS_POKE : 0);
 
         /*
          * Wait and return.
@@ -734,10 +748,10 @@ VMMR3DECL(int) VMR3ReqQueue(PVMREQ pReq, unsigned cMillies)
     }
     else
     {
-        Assert(pUVMCPU);
+        Assert(pUVCpu);
 
         /*
-         * The requester was EMT, just execute it.
+         * The requester was an EMT, just execute it.
          */
         pReq->enmState = VMREQSTATE_QUEUED;
         rc = vmR3ReqProcessOneU(pUVM, pReq);
@@ -784,8 +798,6 @@ VMMR3DECL(int) VMR3ReqWait(PVMREQ pReq, unsigned cMillies)
      */
     PUVM pUVM = pReq->pUVM;
     NOREF(pUVM);
-    AssertMsg(!pUVM->pVM || !VMMR3LockIsOwner(pUVM->pVM),
-              ("Waiting for EMT to process a request, but we own the global VM lock!?!?!?!\n"));
 
     /*
      * Wait on the package.
@@ -821,16 +833,20 @@ VMMR3DECL(int) VMR3ReqWait(PVMREQ pReq, unsigned cMillies)
  * @returns VBox status code.
  *
  * @param   pUVM            Pointer to the user mode VM structure.
- * @param   enmDest         Destination of the request packet (global or per VCPU).
+ * @param   idDstCpu        Pass VMCPUID_ANY to process the common request queue
+ *                          and the CPU ID for a CPU specific one. In the latter
+ *                          case the calling thread must be the EMT of that CPU.
+ *
+ * @note    SMP safe (multiple EMTs trying to satisfy VM_FF_REQUESTs).
  */
-VMMR3DECL(int) VMR3ReqProcessU(PUVM pUVM, VMREQDEST enmDest)
+VMMR3DECL(int) VMR3ReqProcessU(PUVM pUVM, VMCPUID idDstCpu)
 {
-    LogFlow(("VMR3ReqProcessU: (enmVMState=%d) enmDest=%d\n", pUVM->pVM ? pUVM->pVM->enmVMState : VMSTATE_CREATING, enmDest));
+    LogFlow(("VMR3ReqProcessU: (enmVMState=%d) idDstCpu=%d\n", pUVM->pVM ? pUVM->pVM->enmVMState : VMSTATE_CREATING, idDstCpu));
 
     /*
      * Process loop.
      *
-     * We do not repeat the outer loop if we've got an informationtional status code
+     * We do not repeat the outer loop if we've got an informational status code
      * since that code needs processing by our caller.
      */
     int rc = VINF_SUCCESS;
@@ -839,8 +855,8 @@ VMMR3DECL(int) VMR3ReqProcessU(PUVM pUVM, VMREQDEST enmDest)
         /*
          * Get pending requests.
          */
-        void *volatile *ppReqs;
-        if (enmDest == VMREQDEST_ANY)
+        void * volatile *ppReqs;
+        if (idDstCpu == VMCPUID_ANY)
         {
             ppReqs = (void * volatile *)&pUVM->vm.s.pReqs;
             if (RT_LIKELY(pUVM->pVM))
@@ -848,11 +864,12 @@ VMMR3DECL(int) VMR3ReqProcessU(PUVM pUVM, VMREQDEST enmDest)
         }
         else
         {
-            ppReqs = (void * volatile *)&pUVM->aCpus[enmDest].vm.s.pReqs;
+            Assert(idDstCpu < pUVM->cCpus);
+            Assert(pUVM->aCpus[idDstCpu].vm.s.NativeThreadEMT == RTThreadNativeSelf());
+            ppReqs = (void * volatile *)&pUVM->aCpus[idDstCpu].vm.s.pReqs;
             if (RT_LIKELY(pUVM->pVM))
-                VMCPU_FF_CLEAR(pUVM->pVM, enmDest, VM_FF_REQUEST);
+                VMCPU_FF_CLEAR(&pUVM->pVM->aCpus[idDstCpu], VMCPU_FF_REQUEST);
         }
-
         PVMREQ pReqs = (PVMREQ)ASMAtomicXchgPtr(ppReqs, NULL);
         if (!pReqs)
             break;
