@@ -1,33 +1,31 @@
-/* $Id: kLdrMod.c 26 2009-02-19 01:06:09Z bird $ */
+/* $Id: kLdrMod.c 30 2009-07-01 20:54:59Z bird $ */
 /** @file
  * kLdr - The Module Interpreter.
  */
 
 /*
- * Copyright (c) 2006-2007 knut st. osmundsen <bird-kStuff-spam@anduin.net>
+ * Copyright (c) 2006-2007 Knut St. Osmundsen <bird-kStuff-spamix@anduin.net>
  *
- * This file is part of kStuff.
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
  *
- * kStuff is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
  *
- * In addition to the permissions in the GNU Lesser General Public
- * License, you are granted unlimited permission to link the compiled
- * version of this file into combinations with other programs, and to
- * distribute those combinations without any restriction coming from
- * the use of this file.
- *
- * kStuff is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with kStuff; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
  */
 
 /*******************************************************************************
@@ -40,8 +38,6 @@
 #if 1 /* testing headers */
 # include <k/kLdrFmts/pe.h>
 # include <k/kLdrFmts/lx.h>
-# include <k/kLdrFmts/elf32.h>
-# include <k/kLdrFmts/elf64.h>
 # include <k/kLdrFmts/mach-o.h>
 #endif
 
